@@ -4,8 +4,6 @@
 //
 // Code available from: http://www.veripool.org/verilator
 //
-// AUTHORS: Wilson Snyder with Paul Wasson, Duane Gabli
-//
 //*************************************************************************
 //
 // Copyright 2003-2012 by Wilson Snyder.  This program is free software; you can
@@ -53,6 +51,9 @@ bool AstUser3InUse::s_userBusy=false;
 bool AstUser4InUse::s_userBusy=false;
 bool AstUser5InUse::s_userBusy=false;
 
+int AstNodeDType::s_uniqueNum = 0;
+
+
 //######################################################################
 // V3AstType
 
@@ -72,14 +73,12 @@ void AstNode::init() {
     m_op3p = NULL;
     m_op4p = NULL;
     m_iterpp = NULL;
+    m_dtypep = NULL;
     m_clonep = NULL;
     m_cloneCnt = 0;
     // Attributes
-    m_numeric = (int)AstNumeric::UNSIGNED;
     m_didWidth = false;
     m_doingWidth = false;
-    m_width = 0;
-    m_widthMin = 0;
     m_user1p = NULL;
     m_user1Cnt = 0;
     m_user2p = NULL;
@@ -229,15 +228,6 @@ string AstNode::quoteName(const string& namein) {
     return out;
 }
 
-int AstNode::widthPow2() const {
-    // I.e.  width 30 returns 32, width 32 returns 32.
-    uint32_t width = this->width();
-    for (int p2=30; p2>=0; p2--) {
-	if (width > (1UL<<p2)) return (1UL<<(p2+1));
-    }
-    return 1;
-}
-
 //######################################################################
 // Insertion
 
@@ -768,13 +758,16 @@ void AstNode::iterateAndNext(AstNVisitor& v, AstNUser* vup) {
     // Future versions of this function may require the node to have a back to be iterated;
     // there's no lower level reason yet though the back must exist.
     AstNode* nodep=this;
+#ifdef VL_DEBUG  // Otherwise too hot of a function for debug
     if (VL_UNLIKELY(nodep && !nodep->m_backp)) nodep->v3fatalSrc("iterateAndNext node has no back");
+#endif
     while (nodep) {
-	AstNode* niterp = nodep;
+	AstNode* niterp = nodep;  // This address may get stomped via m_iterpp if the node is edited
 	ASTNODE_PREFETCH(nodep->m_nextp);
 	niterp->m_iterpp = &niterp;
 	niterp->accept(v, vup);
 	// accept may do a replaceNode and change niterp on us...
+	//if (niterp != nodep) UINFO(1,"iterateAndNext edited "<<(void*)nodep<<" now into "<<(void*)niterp<<endl);  // niterp maybe NULL, so need cast
 	if (!niterp) return;
 	niterp->m_iterpp = NULL;
 	if (VL_UNLIKELY(niterp!=nodep)) { // Edited it
@@ -856,6 +849,9 @@ AstNode* AstNode::acceptSubtreeReturnEdits(AstNVisitor& v, AstNUser* vup) {
 void AstNode::cloneRelinkTree() {
     if (!this) return;
     for (AstNode* nodep=this; nodep; nodep=nodep->m_nextp) {
+	if (m_dtypep && m_dtypep->clonep()) {
+	    m_dtypep = m_dtypep->clonep()->castNodeDType();
+	}
 	nodep->cloneRelink();
 	nodep->m_op1p->cloneRelinkTree();
 	nodep->m_op2p->cloneRelinkTree();
@@ -876,8 +872,7 @@ bool AstNode::sameTreeIter(AstNode* node2p, bool ignNext) {
     if (this==NULL && node2p==NULL) return true;
     if (this==NULL || node2p==NULL) return false;
     if (this->type() != node2p->type()
-	|| this->width() != node2p->width()
-	|| this->numeric() != node2p->numeric()
+	|| this->dtypep() != node2p->dtypep()
 	|| !this->same(node2p)) {
 	return false;
     }
@@ -954,15 +949,19 @@ void AstNode::dumpGdb() {  // For GDB only
     dumpGdbHeader();
     cout<<"  "; dump(cout); cout<<endl;
 }
+void AstNode::dumpGdbHeader() const {  // For GDB only
+    if (!this) { cout<<"This=NULL"<<endl; return; }
+    dumpPtrs(cout);
+    cout<<"  Fileline = "<<fileline()<<endl;
+}
 void AstNode::dumpTreeGdb() {  // For GDB only
     if (!this) { cout<<"This=NULL"<<endl; return; }
     dumpGdbHeader();
     dumpTree(cout);
 }
-void AstNode::dumpGdbHeader() const {  // For GDB only
-    if (!this) { cout<<"This=NULL"<<endl; return; }
-    dumpPtrs(cout);
-    cout<<"  Fileline = "<<fileline()<<endl;
+void AstNode::dumpTreeFileGdb(const char* filenamep) {  // For GDB only
+    string filename = filenamep ? filenamep : v3Global.debugFilename("debug.tree",98);
+    v3Global.rootp()->dumpTreeFile(filename);
 }
 
 void AstNode::dumpPtrs(ostream& os) const {
@@ -1011,9 +1010,10 @@ void AstNode::dumpTreeAndNext(ostream& os, const string& indent, int maxDepth) {
 void AstNode::dumpTreeFile(const string& filename, bool append) {
     if (v3Global.opt.dumpTree()) {
 	{   // Write log & close
+	    UINFO(2,"Dumping "<<filename<<endl);
 	    const auto_ptr<ofstream> logsp (V3File::new_ofstream(filename, append));
 	    if (logsp->fail()) v3fatalSrc("Can't write "<<filename);
-	    *logsp<<"Tree Dump from <e"<<dec<<editCountLast()<<">";
+	    *logsp<<"Verilator Tree Dump (format 0x3900) from <e"<<dec<<editCountLast()<<">";
 	    *logsp<<" to <e"<<dec<<editCountGbl()<<">"<<endl;
 	    if (editCountGbl()==editCountLast()
 		&& !(v3Global.opt.dumpTree()>=9)) {
@@ -1049,6 +1049,61 @@ void AstNode::v3errorEnd(ostringstream& str) const {
     }
 }
 
+string AstNode::warnMore() const {
+    if (this) return this->fileline()->warnMore();
+    else return V3Error::warnMore();
+}
+
+//======================================================================
+// Data type conversion
+
+void AstNode::dtypeChgSigned(bool flag) {
+    if (!dtypep()) this->v3fatalSrc("No dtype when changing to (un)signed");
+    dtypeChgWidthSigned(dtypep()->width(), dtypep()->widthMin(),
+			flag ? AstNumeric::SIGNED : AstNumeric::UNSIGNED);
+}
+void AstNode::dtypeChgWidth(int width, int widthMin) {
+    if (!dtypep()) this->v3fatalSrc("No dtype when changing width");  // Use ChgWidthSigned(...UNSIGNED) otherwise
+    dtypeChgWidthSigned(width, widthMin, dtypep()->numeric());
+}
+
+void AstNode::dtypeChgWidthSigned(int width, int widthMin, bool issigned) {
+    AstNumeric numeric = issigned ? AstNumeric::SIGNED : AstNumeric::UNSIGNED;
+    if (!dtypep()) {
+	// We allow dtypep() to be null, as before/during widthing dtypes are not resolved
+	dtypeSetLogicSized(width, widthMin, numeric);
+    } else {
+	if (width==dtypep()->width()
+	    && widthMin==dtypep()->widthMin()
+	    && numeric==dtypep()->numeric()) return;  // Correct already
+	// FUTURE: We may be pointing at a two state data type, and this may
+	// convert it to logic.  Since the AstVar remains correct, we
+	// work OK but this assumption may break in the future.
+	// Note we can't just clone and do a widthForce, as if it's a BasicDType
+	// the msb() indications etc will be incorrect.
+	dtypeSetLogicSized(width, widthMin, numeric);
+    }
+}
+
+AstNodeDType* AstNode::findBasicDType(AstBasicDTypeKwd kwd) const {
+    // For 'simple' types we use the global directory.  These are all unsized.
+    // More advanced types land under the module/task/etc
+    return v3Global.rootp()->typeTablep()
+	->findBasicDType(fileline(), kwd);
+}
+AstNodeDType* AstNode::findBitDType(int width, int widthMin, AstNumeric numeric) const {
+    return v3Global.rootp()->typeTablep()
+	->findLogicBitDType(fileline(), AstBasicDTypeKwd::BIT, width, widthMin, numeric);
+}
+AstNodeDType* AstNode::findLogicDType(int width, int widthMin, AstNumeric numeric) const {
+    return v3Global.rootp()->typeTablep()
+	->findLogicBitDType(fileline(), AstBasicDTypeKwd::LOGIC, width, widthMin, numeric);
+}
+AstBasicDType* AstNode::findInsertSameDType(AstBasicDType* nodep) {
+    return v3Global.rootp()->typeTablep()
+	->findInsertSameDType(nodep);
+}
+
 //######################################################################
 // AstNVisitor
 
