@@ -4,8 +4,6 @@
 //
 // Code available from: http://www.veripool.org/verilator
 //
-// AUTHORS: Wilson Snyder with Paul Wasson, Duane Gabli
-//
 //*************************************************************************
 //
 // Copyright 2003-2012 by Wilson Snyder.  This program is free software; you can
@@ -48,6 +46,28 @@ int AstNodeSel::bitConst() const {
     AstConst* constp=bitp()->castConst(); return (constp?constp->toSInt():0);
 }
 
+void AstNodeClassDType::repairMemberCache() {
+    clearCache();
+    for (AstMemberDType* itemp = membersp(); itemp; itemp=itemp->nextp()->castMemberDType()) {
+	if (m_members.find(itemp->name())!=m_members.end()) { itemp->v3error("Duplicate declaration of member name: "<<itemp->prettyName()); }
+	else m_members.insert(make_pair(itemp->name(), itemp));
+    }
+}
+
+bool AstNodeClassDType::broken() const {
+    set<AstMemberDType*> exists;
+    for (AstMemberDType* itemp = membersp(); itemp; itemp=itemp->nextp()->castMemberDType()) {
+	exists.insert(itemp);
+    }
+    for (MemberNameMap::const_iterator it=m_members.begin(); it!=m_members.end(); ++it) {
+	if (exists.find(it->second) == exists.end()) {
+	    this->v3error("Internal: Structure member broken: "<<it->first);
+	    return true;
+	}
+    }
+    return false;
+}
+
 int AstBasicDType::widthAlignBytes() const {
     if (width()<=8) return 1;
     else if (width()<=16) return 2;
@@ -62,6 +82,22 @@ int AstBasicDType::widthTotalBytes() const {
     else return widthWords()*(VL_WORDSIZE/8);
 }
 
+int AstNodeClassDType::widthTotalBytes() const {
+    if (width()<=8) return 1;
+    else if (width()<=16) return 2;
+    else if (isQuad()) return 8;
+    else return widthWords()*(VL_WORDSIZE/8);
+}
+
+int AstNodeClassDType::widthAlignBytes() const {
+    // Could do max across members but that would be slow,
+    // instead intuit based on total structure size
+    if (width()<=8) return 1;
+    else if (width()<=16) return 2;
+    else if (width()<=32) return 4;
+    else return 8;
+}
+
 bool AstVar::isSigPublic() const {
     return (m_sigPublic || (v3Global.opt.allPublic() && !isTemp() && !isGenVar()));
 }
@@ -75,16 +111,26 @@ bool AstVar::isScBv() const {
 }
 
 void AstVar::combineType(AstVarType type) {
+    // These flags get combined with the existing settings of the flags.
+    // We don't test varType for certain types, instead set flags since
+    // when we combine wires cross-hierarchy we need a union of all characteristics.
     if (type == AstVarType::SUPPLY0) type = AstVarType::WIRE;
     if (type == AstVarType::SUPPLY1) type = AstVarType::WIRE;
     m_varType=type; 	// For debugging prints only
     // These flags get combined with the existing settings of the flags.
     if (type==AstVarType::INPUT || type==AstVarType::INOUT)
 	m_input = true;
-    if (type==AstVarType::OUTPUT || type==AstVarType::INOUT)
+    if (type==AstVarType::OUTPUT || type==AstVarType::INOUT) {
 	m_output = true;
-    if (type==AstVarType::INOUT || type==AstVarType::TRIWIRE)
+	m_declOutput = true;
+    }
+    if (type==AstVarType::INOUT || type==AstVarType::TRIWIRE
+	|| type==AstVarType::TRI0 || type==AstVarType::TRI1)
 	m_tristate = true;
+    if (type==AstVarType::TRI0)
+	m_isPulldown = true;
+    if (type==AstVarType::TRI1)
+	m_isPullup = true;
 }
 
 string AstVar::verilogKwd() const {
@@ -277,6 +323,14 @@ AstNodeDType* AstNodeDType::dtypeDimensionp(int dimension) {
 	    }
 	    return NULL;
 	}
+	else if (AstNodeClassDType* adtypep = dtypep->castNodeClassDType()) {
+	    if (adtypep->packed()) {
+		if ((dim++) == dimension) {
+		    return adtypep;
+		}
+	    }
+	    return NULL;
+	}
 	// Node no ->next in loop; use continue where necessary
 	break;
     }
@@ -318,6 +372,15 @@ pair<uint32_t,uint32_t> AstNodeDType::dimensions() {
     return make_pair(packed, unpacked);
 }
 
+int AstNodeDType::widthPow2() const {
+    // I.e.  width 30 returns 32, width 32 returns 32.
+    uint32_t width = this->width();
+    for (int p2=30; p2>=0; p2--) {
+	if (width > (1UL<<p2)) return (1UL<<(p2+1));
+    }
+    return 1;
+}
+
 // Special operators
 int AstArraySel::dimension(AstNode* nodep) {
     // How many dimensions is this reference from the base variable?
@@ -337,7 +400,7 @@ AstNode* AstArraySel::baseFromp(AstNode* nodep) {	///< What is the base variable
     while (nodep) {
 	if (nodep->castArraySel()) { nodep=nodep->castArraySel()->fromp(); continue; }
 	else if (nodep->castSel()) { nodep=nodep->castSel()->fromp(); continue; }
-	// AstNodeSelPre stashes the associated variable under a ATTROF so it isn't constified
+	// AstNodeSelPre stashes the associated variable under a ATTROF of AstAttrType::VAR_BASE/MEMBER_BASE so it isn't constified
 	else if (nodep->castAttrOf()) { nodep=nodep->castAttrOf()->fromp(); continue; }
 	else if (nodep->castNodePreSel()) {
 	    if (nodep->castNodePreSel()->attrp()) {
@@ -432,6 +495,87 @@ bool AstSenTree::hasCombo() {
     return false;
 }
 
+void AstTypeTable::clearCache() {
+    // When we mass-change widthMin in V3WidthCommit, we need to correct the table.
+    // Just clear out the maps; the search functions will be used to rebuild the map
+    for (int i=0; i<(int)(AstBasicDTypeKwd::_ENUM_MAX); ++i) {
+	m_basicps[i] = NULL;
+    }
+    for (int isbit=0; isbit<_IDX0_MAX; ++isbit) {
+	for (int numer=0; numer<AstNumeric::_ENUM_MAX; ++numer) {
+	    LogicMap& mapr = m_logicMap[isbit][numer];
+	    mapr.clear();
+	}
+    }
+    m_detailedMap.clear();
+    // Clear generic()'s so dead detection will work
+    for (AstNode* nodep = typesp(); nodep; nodep=nodep->nextp()) {
+	if (AstBasicDType* bdtypep = nodep->castBasicDType()) {
+	    bdtypep->generic(false);
+	}
+    }
+}
+
+void AstTypeTable::repairCache() {
+    // After we mass-change widthMin in V3WidthCommit, we need to correct the table.
+    clearCache();
+    for (AstNode* nodep = typesp(); nodep; nodep=nodep->nextp()) {
+	if (AstBasicDType* bdtypep = nodep->castBasicDType()) {
+	    (void)findInsertSameDType(bdtypep);
+	}
+    }
+}
+
+AstBasicDType* AstTypeTable::findBasicDType(FileLine* fl, AstBasicDTypeKwd kwd) {
+    if (m_basicps[kwd]) return m_basicps[kwd];
+    //
+    AstBasicDType* new1p = new AstBasicDType(fl, kwd);
+    // Because the detailed map doesn't update this map,
+    // check the detailed map for this same node
+    // Also adds this new node to the detailed map
+    AstBasicDType* newp = findInsertSameDType(new1p);
+    if (newp != new1p) new1p->deleteTree();
+    else addTypesp(newp);
+    //
+    m_basicps[kwd] = newp;
+    return newp;
+}
+
+AstBasicDType* AstTypeTable::findLogicBitDType(FileLine* fl, AstBasicDTypeKwd kwd,
+					       int width, int widthMin, AstNumeric numeric) {
+    int idx = IDX0_LOGIC;
+    if (kwd == AstBasicDTypeKwd::LOGIC) idx = IDX0_LOGIC;
+    else if (kwd == AstBasicDTypeKwd::BIT) idx = IDX0_BIT;
+    else fl->v3fatalSrc("Bad kwd for findLogicBitDType");
+    pair<int,int> widths = make_pair(width,widthMin);
+    LogicMap& mapr = m_logicMap[idx][(int)numeric];
+    LogicMap::const_iterator it = mapr.find(widths);
+    if (it != mapr.end()) return it->second;
+    //
+    AstBasicDType* new1p = new AstBasicDType(fl, AstBasicDTypeKwd::BIT, numeric, width, widthMin);
+    // Because the detailed map doesn't update this map,
+    // check the detailed map for this same node, and if found update this map
+    // Also adds this new node to the detailed map
+    AstBasicDType* newp = findInsertSameDType(new1p);
+    if (newp != new1p) new1p->deleteTree();
+    else addTypesp(newp);
+    //
+    mapr.insert(make_pair(widths,newp));
+    return newp;
+}
+
+AstBasicDType* AstTypeTable::findInsertSameDType(AstBasicDType* nodep) {
+    VBasicTypeKey key (nodep->width(), nodep->widthMin(), nodep->numeric(),
+		       nodep->keyword(), nodep->nrange());
+    DetailedMap& mapr = m_detailedMap;
+    DetailedMap::const_iterator it = mapr.find(key);
+    if (it != mapr.end()) return it->second;
+    mapr.insert(make_pair(key,nodep));
+    nodep->generic(true);
+    // No addTypesp; the upper function that called new() is responsible for adding
+    return nodep;
+}
+
 //======================================================================
 // Special walking tree inserters
 
@@ -502,11 +646,21 @@ void AstNode::dump(ostream& str) {
 	//<<" "<<(void*)this->m_backp
        <<" <e"<<dec<<editCount()
        <<((editCount()>=editCountLast())?"#>":">")
-       <<" {"<<fileline()->filenameLetters()<<dec<<fileline()->lineno()<<"}"
-       <<" "<<(isSigned()?"s":"")
-       <<(isDouble()?"d":"")
-       <<"w"<<(widthSized()?"":"u")<<width();
-    if (!widthSized()) str<<"/"<<widthMin();
+       <<" {"<<fileline()->filenameLetters()<<dec<<fileline()->lineno()<<"}";
+    if (user1p()) str<<" u1="<<(void*)user1p();
+    if (user2p()) str<<" u2="<<(void*)user2p();
+    if (user3p()) str<<" u3="<<(void*)user3p();
+    if (user4p()) str<<" u4="<<(void*)user4p();
+    if (user5p()) str<<" u5="<<(void*)user5p();
+    if (hasDType()) {
+	if (dtypep()==this) str<<" @dt="<<"this@";
+	else str<<" @dt="<<(void*)dtypep()<<"@";  // Final @ so less likely to by accident think it's nodep
+	if (AstNodeDType* dtp = dtypep()) {
+	    dtp->dumpSmall(str);
+	}
+    } else { // V3Broken will throw an error
+	if (dtypep()) str<<" %Error-dtype-exp=null,got="<<(void*)dtypep();
+    }
     if (name()!="") str<<"  "<<AstNode::quoteName(name());
 }
 
@@ -526,7 +680,6 @@ void AstBasicDType::dump(ostream& str) {
     this->AstNodeDType::dump(str);
     str<<" kwd="<<keyword().ascii();
     if (isRanged() && !rangep()) str<<" range=["<<left()<<":"<<right()<<"]";
-    if (implicit()) str<<" [IMPLICIT]";
 }
 void AstCCast::dump(ostream& str) {
     this->AstNode::dump(str);
@@ -572,8 +725,87 @@ void AstRefDType::dump(ostream& str) {
     if (defp()) { str<<" -> "; defp()->dump(str); }
     else { str<<" -> UNLINKED"; }
 }
+void AstNodeClassDType::dump(ostream& str) {
+    this->AstNode::dump(str);
+    if (packed()) str<<" [PACKED]";
+}
+void AstNodeDType::dump(ostream& str) {
+    this->AstNode::dump(str);
+    if (generic()) str<<" [GENERIC]";
+    if (AstNodeDType* dtp = virtRefDTypep()) {
+	str<<" refdt="<<(void*)(dtp);
+	dtp->dumpSmall(str);
+    }
+}
+void AstNodeDType::dumpSmall(ostream& str) {
+    str<<"("
+       <<(generic()?"G/":"")
+       <<((isSigned()&&!isDouble())?"s":"")
+       <<(isNosign()?"n":"")
+       <<(isDouble()?"d":"")
+       <<"w"<<(widthSized()?"":"u")<<width();
+    if (!widthSized()) str<<"/"<<widthMin();
+    str<<")";
+}
+void AstNodeModule::dump(ostream& str) {
+    this->AstNode::dump(str);
+    str<<"  L"<<level();
+    if (modPublic()) str<<" [P]";
+    if (inLibrary()) str<<" [LIB]";
+    if (dead()) str<<" [DEAD]";
+}
+void AstPackageImport::dump(ostream& str) {
+    this->AstNode::dump(str);
+    str<<" -> "<<packagep();
+}
+void AstTypeTable::dump(ostream& str) {
+    this->AstNode::dump(str);
+    for (int i=0; i<(int)(AstBasicDTypeKwd::_ENUM_MAX); ++i) {
+	if (AstBasicDType* subnodep=m_basicps[i]) {
+	    str<<endl;  // Newline from caller, so newline first
+	    str<<"\t\t"<<setw(8)<<AstBasicDTypeKwd(i).ascii();
+	    str<<"  -> ";
+	    subnodep->dump(str);
+	}
+    }
+    for (int isbit=0; isbit<2; ++isbit) {
+	for (int issigned=0; issigned<AstNumeric::_ENUM_MAX; ++issigned) {
+	    LogicMap& mapr = m_logicMap[isbit][issigned];
+	    for (LogicMap::const_iterator it = mapr.begin(); it != mapr.end(); ++it) {
+		AstBasicDType* dtypep = it->second;
+		str<<endl;  // Newline from caller, so newline first
+		stringstream nsstr;
+		nsstr<<(isbit?"bw":"lw")
+		     <<it->first.first<<"/"<<it->first.second;
+		str<<"\t\t"<<setw(8)<<nsstr.str();
+		if (issigned) str<<" s"; else str<<" u";
+		str<<"  ->  ";
+		dtypep->dump(str);
+	    }
+	}
+    }
+    {
+	DetailedMap& mapr = m_detailedMap;
+	for (DetailedMap::const_iterator it = mapr.begin(); it != mapr.end(); ++it) {
+	    AstBasicDType* dtypep = it->second;
+	    str<<endl;  // Newline from caller, so newline first
+	    stringstream nsstr;
+	    str<<"\t\tdetailed  ->  ";
+	    dtypep->dump(str);
+	}
+    }
+    // Note get newline from caller too.
+}
+
+void AstVarScope::dump(ostream& str) {
+    this->AstNode::dump(str);
+    if (isCircular()) str<<" [CIRC]";
+    if (varp()) { str<<" -> "; varp()->dump(str); }
+    else { str<<" ->UNLINKED"; }
+}
 void AstVarXRef::dump(ostream& str) {
     this->AstNode::dump(str);
+    if (packagep()) { str<<" pkg="<<(void*)packagep(); }
     if (lvalue()) str<<" [LV] => ";
     else          str<<" [RV] <- ";
     str<<dotted()<<". - ";
@@ -582,27 +814,9 @@ void AstVarXRef::dump(ostream& str) {
     else if (varp()) { varp()->dump(str); }
     else { str<<"UNLINKED"; }
 }
-void AstNodeDType::dump(ostream& str) {
-    this->AstNode::dump(str);
-}
-void AstNodeModule::dump(ostream& str) {
-    this->AstNode::dump(str);
-    str<<"  L"<<level();
-    if (modPublic()) str<<" [P]";
-    if (inLibrary()) str<<" [LIB]";
-}
-void AstPackageImport::dump(ostream& str) {
-    this->AstNode::dump(str);
-    str<<" -> "<<packagep();
-}
-void AstVarScope::dump(ostream& str) {
-    this->AstNode::dump(str);
-    if (isCircular()) str<<" [CIRC]";
-    if (varp()) { str<<" -> "; varp()->dump(str); }
-    else { str<<" ->UNLINKED"; }
-}
 void AstVarRef::dump(ostream& str) {
     this->AstNode::dump(str);
+    if (packagep()) { str<<" pkg="<<(void*)packagep(); }
     if (lvalue()) str<<" [LV] => ";
     else          str<<" [RV] <- ";
     if (varScopep()) { varScopep()->dump(str); }
@@ -619,6 +833,8 @@ void AstVar::dump(ostream& str) {
 	else if (isOutput()) str<<" [O]";
     }
     if (isConst()) str<<" [CONST]";
+    if (isPullup()) str<<" [PULLUP]";
+    if (isPulldown()) str<<" [PULLDOWN]";
     if (isUsedClock()) str<<" [CLK]";
     if (isSigPublic()) str<<" [P]";
     if (isUsedLoopIdx()) str<<" [LOOP]";
@@ -640,6 +856,11 @@ void AstSenItem::dump(ostream& str) {
 void AstParseRef::dump(ostream& str) {
     this->AstNode::dump(str);
     str<<" ["<<expect().ascii()<<"]";
+    if (start()) str<<" [START]";
+}
+void AstDot::dump(ostream& str) {
+    this->AstNode::dump(str);
+    if (start()) str<<" [START]";
 }
 void AstActive::dump(ostream& str) {
     this->AstNode::dump(str);
@@ -649,6 +870,7 @@ void AstActive::dump(ostream& str) {
 }
 void AstNodeFTaskRef::dump(ostream& str) {
     this->AstNode::dump(str);
+    if (packagep()) { str<<" pkg="<<(void*)packagep(); }
     str<<" -> ";
     if (dotted()!="") { str<<dotted()<<". - "; }
     if (taskp()) { taskp()->dump(str); }
@@ -665,7 +887,8 @@ void AstNodeFTask::dump(ostream& str) {
 void AstBegin::dump(ostream& str) {
     this->AstNode::dump(str);
     if (unnamed()) str<<" [UNNAMED]";
-    if (hidden()) str<<" [HIDDEN]";
+    if (generate()) str<<" [GEN]";
+    if (genforp()) str<<" [GENFOR]";
 }
 void AstCoverDecl::dump(ostream& str) {
     this->AstNode::dump(str);
