@@ -4,8 +4,6 @@
 //
 // Code available from: http://www.veripool.org/verilator
 //
-// AUTHORS: Wilson Snyder with Paul Wasson, Duane Gabli
-//
 //*************************************************************************
 //
 // Copyright 2003-2012 by Wilson Snyder.  This program is free software; you can
@@ -362,7 +360,7 @@ private:
     CdcEitherVertex* traceAsyncRecurse(CdcEitherVertex* vertexp, bool mark) {
 	// First pass: Return vertex of any hazardous stuff attached, or NULL if OK
 	// If first pass returns true, second pass calls asyncPath() on appropriate nodes
-	if (vertexp->user()>=m_userGeneration) return false;   // Processed - prevent loop
+	if (vertexp->user()>=m_userGeneration) return NULL;   // Processed - prevent loop
 	vertexp->user(m_userGeneration);
 
 	CdcEitherVertex* mark_outp = NULL;
@@ -385,7 +383,7 @@ private:
 		    CdcEitherVertex* eFromVertexp = (CdcEitherVertex*)edgep->fromp();
 		    eFromVertexp->asyncPath(true);
 		}
-		return false;
+		return NULL;
 	    }
 	    // Also ok if from flop, but partially trace the flop so more obvious to users
 	    if (vvertexp->fromFlop()) {
@@ -393,7 +391,7 @@ private:
 		    CdcEitherVertex* eFromVertexp = (CdcEitherVertex*)edgep->fromp();
 		    eFromVertexp->asyncPath(true);
 		}
-		return false;
+		return NULL;
 	    }
 	}
 
