@@ -4,8 +4,6 @@
 //
 // Code available from: http://www.veripool.org/verilator
 //
-// AUTHORS: Wilson Snyder with Paul Wasson, Duane Gabli
-//
 //*************************************************************************
 //
 // Copyright 2003-2012 by Wilson Snyder.  This program is free software; you can
@@ -31,8 +29,12 @@
 
 class V3Dead {
 public:
+    // Modules, no vars/dtypes
+    static void deadifyModules(AstNetlist* nodep);
+    // Modules, Data types
+    static void deadifyDTypes(AstNetlist* nodep);
     // Everything that's possible
-    static void deadifyAll(AstNetlist* nodep, bool elimUserVars);
+    static void deadifyAll(AstNetlist* nodep);
 };
 
 #endif // Guard
