@@ -4,8 +4,6 @@
 //
 // Code available from: http://www.veripool.org/verilator
 //
-// AUTHORS: Wilson Snyder with Paul Wasson, Duane Gabli
-//
 //*************************************************************************
 //
 // Copyright 2003-2012 by Wilson Snyder.  This program is free software; you can
@@ -133,7 +131,7 @@ private:
 	} else {
 	    if (width==0) {
 		varp = new AstVar (oldvarscp->fileline(), AstVarType::BLOCKTEMP, name, oldvarscp->varp());
-		varp->widthSignedFrom(oldvarscp);
+		varp->dtypeFrom(oldvarscp);
 	    } else { // Used for vset and dimensions, so can zero init
 		varp = new AstVar (oldvarscp->fileline(), AstVarType::BLOCKTEMP, name, VFlagBitPacked(), width);
 	    }
@@ -158,9 +156,9 @@ private:
 	if (oldactivep->sensesp() != m_activep->sensesp()) {
 	    if (!varrefp->varp()->fileline()->warnIsOff(V3ErrorCode::MULTIDRIVEN)
 		&& !varrefp->varp()->user2()) {
-		varrefp->varp()->v3warn(MULTIDRIVEN,"Signal has multiple driving blocks: "<<varrefp->varp()->prettyName());
-		varrefp->v3warn(MULTIDRIVEN,"... Location of first driving block");
-		oldactivep->v3warn(MULTIDRIVEN,"... Location of other driving block");
+		varrefp->varp()->v3warn(MULTIDRIVEN,"Signal has multiple driving blocks: "<<varrefp->varp()->prettyName()<<endl
+					<<varrefp->warnMore()<<"... Location of first driving block"<<endl
+					<<oldactivep->warnMore()<<"... Location of other driving block");
 		varrefp->varp()->user2(true);
 	    }
 	    UINFO(4,"AssignDupDlyVar: "<<varrefp<<endl);
