@@ -4,8 +4,6 @@
 //
 // Code available from: http://www.veripool.org/verilator
 //
-// AUTHORS: Wilson Snyder with Paul Wasson, Duane Gabli
-//
 //*************************************************************************
 //
 // Copyright 2004-2012 by Wilson Snyder.  This program is free software; you can
@@ -384,6 +382,7 @@ class EmitVBaseVisitor : public EmitCBaseVisitor {
 	//	%l	lhsp - if appropriate
 	//	%r	rhsp - if appropriate
 	//	%t	thsp - if appropriate
+	//	%d	dtypep - if appropriate
 	//	%k	Potential line break
 	bool inPct = false;
 	putbs("");
@@ -413,6 +412,11 @@ class EmitVBaseVisitor : public EmitCBaseVisitor {
 		    else thsp->iterateAndNext(*this);
 		    break;
 		}
+		case 'd': {
+		    if (!nodep->dtypep()) { nodep->v3fatalSrc("emitVerilog() references undef node"); }
+		    else nodep->dtypep()->iterateAndNext(*this);
+		    break;
+		}
 		default:
 		    nodep->v3fatalSrc("Unknown emitVerilog format code: %"<<pos[0]);
 		    break;
@@ -495,7 +499,20 @@ class EmitVBaseVisitor : public EmitCBaseVisitor {
     }
     virtual void visit(AstArrayDType* nodep, AstNUser*) {
 	nodep->subDTypep()->iterateAndNext(*this);
-	nodep->arrayp()->iterateAndNext(*this);
+	nodep->rangep()->iterateAndNext(*this);
+    }
+    virtual void visit(AstNodeClassDType* nodep, AstNUser*) {
+	puts(nodep->verilogKwd()+" ");
+	if (nodep->packed()) puts("packed ");
+	puts("\n");
+	nodep->membersp()->iterateAndNext(*this);
+	puts("}");
+    }
+    virtual void visit(AstMemberDType* nodep, AstNUser*) {
+	nodep->subDTypep()->iterateAndNext(*this);
+	puts(" ");
+	puts(nodep->name());
+	puts("}");
     }
     virtual void visit(AstNodeFTaskRef* nodep, AstNUser*) {
 	if (nodep->dotted()!="") { putfs(nodep,nodep->dotted()); puts("."); puts(nodep->prettyName()); }
