@@ -4,8 +4,6 @@
 //
 // Code available from: http://www.veripool.org/verilator
 //
-// AUTHORS: Wilson Snyder with Paul Wasson, Duane Gabli
-//
 //*************************************************************************
 //
 // Copyright 2003-2012 by Wilson Snyder.  This program is free software; you can
@@ -302,7 +300,7 @@ private:
     }
     bool readContentsFile(const string& filename, StrList& outl) {
 	int fd = open (filename.c_str(), O_RDONLY);
-	if (!fd) return false;
+	if (fd<0) return false;
 	m_readEof = false;
 	readBlocks(fd, -1, outl);
 	close(fd);
