@@ -4,8 +4,6 @@
 //
 // Code available from: http://www.veripool.org/verilator
 //
-// AUTHORS: Wilson Snyder with Paul Wasson, Duane Gabli
-//
 //*************************************************************************
 //
 // Copyright 2003-2012 by Wilson Snyder.  This program is free software; you can
@@ -277,10 +275,10 @@ void V3Graph::dumpDotFile(const string& filename, bool colorAsSubgraph) {
 
     // Header
     *logp<<"digraph v3graph {\n";
-    *logp<<"\trankdir="<<dotRankDir()<<"\n";
-    *logp<<"\t//size="<<"\"7.5,10\""<<"\n";
-    *logp<<"\tnTITLE\t[fontsize=20 label=\""<<filename<<"\", shape=box, style=bold, color=black];\n";
-    *logp<<"\n";
+    *logp<<"\tgraph\t[label=\""<<filename<<"\",\n";
+    *logp<<"\t\t labelloc=t, labeljust=l,\n";
+    *logp<<"\t\t //size="<<"\"7.5,10\","<<"\n";
+    *logp<<"\t\t rankdir="<<dotRankDir()<<"];\n";
 
     // List of all possible subgraphs
     typedef multimap<string,V3GraphVertex*> SubgraphMmap;
@@ -346,5 +344,5 @@ void V3Graph::dumpDotFile(const string& filename, bool colorAsSubgraph) {
     *logp << "}\n";
     logp->close();
 
-    cout << "dot -Tps -o ~/a.ps "<<filename<<endl;
+    cout << "dot -Tpdf -o ~/a.pdf "<<filename<<endl;
 }
