@@ -4,8 +4,6 @@
 //
 // Code available from: http://www.veripool.org/verilator
 //
-// AUTHORS: Wilson Snyder with Paul Wasson, Duane Gabli
-//
 //*************************************************************************
 //
 // Copyright 2003-2012 by Wilson Snyder.  This program is free software; you can
@@ -114,7 +112,8 @@ private:
 	    // we'll save work, and we can't call pinReconnectSimple in
 	    // this loop as it clone()s itself.
 	    for (AstPin* pinp = nodep->pinsp(); pinp; pinp=pinp->nextp()->castPin()) {
-		V3Inst::pinReconnectSimple(pinp, nodep, m_modp);
+		if (!pinp->exprp()) continue;
+		V3Inst::pinReconnectSimple(pinp, nodep, m_modp, false);
 	    }
 
 	    // Clone original module
@@ -130,6 +129,7 @@ private:
 	    m_modp->addInlinesp(inlinep);  // Must be parsed before any AstCells
 	    // Create assignments to the pins
 	    for (AstPin* pinp = nodep->pinsp(); pinp; pinp=pinp->nextp()->castPin()) {
+		if (!pinp->exprp()) continue;
 		UINFO(6,"     Pin change from "<<pinp->modVarp()<<endl);
 		// Make new signal; even though we'll optimize the interconnect, we
 		// need an alias to trace correctly.  If tracing is disabled, we'll
