@@ -4,8 +4,6 @@
 //
 // Code available from: http://www.veripool.org/verilator
 //
-// AUTHORS: Wilson Snyder with Paul Wasson, Duane Gabli
-//
 //*************************************************************************
 //
 // Copyright 2003-2012 by Wilson Snyder.  This program is free software; you can
@@ -93,18 +91,20 @@ private:
     //  Entire netlist:
     //   AstNodeModule::user1p()	// V3GraphVertex*    Vertex describing this module
     //  Allocated across all readFiles in V3Global::readFiles:
-    //   AstNode::user4p()	// V3SymTable*    Package and typedef symbol names
+    //   AstNode::user4p()	// VSymEnt*    Package and typedef symbol names
     AstUser1InUse	m_inuser1;
 
     // STATE
     V3InFilter*		m_filterp;	// Parser filter
+    V3ParseSym*		m_parseSymp;	// Parser symbol table
 
     // Below state needs to be preserved between each module call.
     AstNodeModule*	m_modp;		// Current module
-    V3SymTable  	m_mods;		// Symbol table of all module names
+    VSymGraph	  	m_mods;		// Symbol table of all module names
     LinkCellsGraph	m_graph;	// Linked graph of all cell interconnects
     LibraryVertex*	m_libVertexp;	// Vertex at root of all libraries
     V3GraphVertex*	m_topVertexp;	// Vertex of top module
+    set<string>		m_declfnWarned;	// Files we issued DECLFILENAME on
 
     static int debug() {
 	static int level = -1;
@@ -157,6 +157,15 @@ private:
 	// Module: Pick up modnames, so we can resolve cells later
 	m_modp = nodep;
 	UINFO(2,"Link Module: "<<nodep<<endl);
+	if (nodep->fileline()->filebasenameNoExt() != nodep->prettyName()
+	    && !v3Global.opt.isLibraryFile(nodep->fileline()->filename())) {
+	    // We only complain once per file, otherwise library-like files have a huge mess of warnings
+	    if (m_declfnWarned.find(nodep->fileline()->filename()) == m_declfnWarned.end()) {
+		m_declfnWarned.insert(nodep->fileline()->filename());
+		nodep->v3warn(DECLFILENAME, "Filename '"<<nodep->fileline()->filebasenameNoExt()
+			      <<"' does not match "<<nodep->typeName()<<" name: "<<nodep->prettyName());
+	    }
+	}
 	bool topMatch = (v3Global.opt.topModule()==nodep->name());
 	if (topMatch) {
 	    m_topVertexp = vertex(nodep);
@@ -176,24 +185,31 @@ private:
 	m_modp = NULL;
     }
 
+    virtual void visit(AstPackageImport* nodep, AstNUser*) {
+	// Package Import: We need to do the package before the use of a package
+	nodep->iterateChildren(*this);
+	if (!nodep->packagep()) nodep->v3fatalSrc("Unlinked package");  // Parser should set packagep
+	new V3GraphEdge(&m_graph, vertex(m_modp), vertex(nodep->packagep()), 1, false);
+    }
+
     virtual void visit(AstCell* nodep, AstNUser*) {
 	// Cell: Resolve its filename.  If necessary, parse it.
 	if (!nodep->modp()) {
 	    UINFO(4,"Link Cell: "<<nodep<<endl);
-	    // Use findIdUpward instead of findIdFlat; it doesn't matter for now
+	    // Use findIdFallback instead of findIdFlat; it doesn't matter for now
 	    // but we might support modules-under-modules someday.
-	    AstNodeModule* modp = m_mods.findIdUpward(nodep->modName())->castNodeModule();
+	    AstNodeModule* modp = m_mods.rootp()->findIdFallback(nodep->modName())->nodep()->castNodeModule();
 	    if (!modp) {
 		// Read-subfile
 		// If file not found, make AstNotFoundModule, rather than error out.
 		// We'll throw the error when we know the module will really be needed.
-		V3Parse parser (v3Global.rootp(), m_filterp);
+		V3Parse parser (v3Global.rootp(), m_filterp, m_parseSymp);
 		parser.parseFile(nodep->fileline(), nodep->modName(), false, "");
 		V3Error::abortIfErrors();
 		// We've read new modules, grab new pointers to their names
 		readModNames();
 		// Check again
-		modp = m_mods.findIdUpward(nodep->modName())->castNodeModule();
+		modp = m_mods.rootp()->findIdFallback(nodep->modName())->nodep()->castNodeModule();
 		if (!modp) {
 		    nodep->v3error("Can't resolve module reference: "<<nodep->modName());
 		}
@@ -227,31 +243,6 @@ private:
 		pinp->unlinkFrBack()->deleteTree(); pinp=NULL;
 	    }
 	}
-	if (nodep->modp() && pinStar) {
-	    // Note what pins exist
-	    UINFO(9,"  CELL .* connect "<<nodep<<endl);
-	    V3SymTable  ports;	// Symbol table of all connected port names
-	    for (AstPin* pinp = nodep->pinsp(); pinp; pinp=pinp->nextp()->castPin()) {
-		if (pinp->name()=="") pinp->v3error("Connect by position is illegal in .* connected cells");
-		if (!ports.findIdFlat(pinp->name())) {
-		    ports.insert(pinp->name(), pinp);
-		}
-	    }
-	    // We search ports, rather than in/out declarations as they aren't resolved yet,
-	    // and it's easier to do it now than in V3Link when we'd need to repeat steps.
-	    for (AstNode* portnodep = nodep->modp()->stmtsp(); portnodep; portnodep=portnodep->nextp()) {
-		if (AstPort* portp = portnodep->castPort()) {
-		    if (!ports.findIdFlat(portp->name())) {
-			UINFO(9,"    need PORT  "<<portp<<endl);
-			// Create any not already connected
-			AstPin* newp = new AstPin(nodep->fileline(),0,portp->name(),
-						  new AstVarRef(nodep->fileline(),portp->name(),false));
-			newp->svImplicit(true);
-			nodep->addPinsp(newp);
-		    }
-		}
-	    }
-	}
 	// Convert unnamed pins to pin number based assignments
 	for (AstPin* pinp = nodep->pinsp(); pinp; pinp=pinp->nextp()->castPin()) {
 	    if (pinp->name()=="") pinp->name("__pinNumber"+cvtToStr(pinp->pinNum()));
@@ -259,9 +250,42 @@ private:
 	for (AstPin* pinp = nodep->paramsp(); pinp; pinp=pinp->nextp()->castPin()) {
 	    if (pinp->name()=="") pinp->name("__paramNumber"+cvtToStr(pinp->pinNum()));
 	}
+	if (nodep->modp()) {
+	    // Note what pins exist
+	    set<string> ports;	// Symbol table of all connected port names
+	    for (AstPin* pinp = nodep->pinsp(); pinp; pinp=pinp->nextp()->castPin()) {
+		if (pinp->name()=="") pinp->v3error("Connect by position is illegal in .* connected cells");
+		if (!pinp->exprp()) pinp->v3warn(PINNOCONNECT,"Cell pin is not connected: "<<pinp->prettyName());
+		if (ports.find(pinp->name()) == ports.end()) {
+		    ports.insert(pinp->name());
+		}
+	    }
+	    // We search ports, rather than in/out declarations as they aren't resolved yet,
+	    // and it's easier to do it now than in V3LinkDot when we'd need to repeat steps.
+	    for (AstNode* portnodep = nodep->modp()->stmtsp(); portnodep; portnodep=portnodep->nextp()) {
+		if (AstPort* portp = portnodep->castPort()) {
+		    if (ports.find(portp->name()) == ports.end()
+			&& ports.find("__pinNumber"+cvtToStr(portp->pinNum())) == ports.end()) {
+			if (pinStar) {
+			    UINFO(9,"    need .* PORT  "<<portp<<endl);
+			    // Create any not already connected
+			    AstPin* newp = new AstPin(nodep->fileline(),0,portp->name(),
+						      new AstVarRef(nodep->fileline(),portp->name(),false));
+			    newp->svImplicit(true);
+			    nodep->addPinsp(newp);
+			} else {  // warn on the CELL that needs it, not the port
+			    nodep->v3warn(PINMISSING, "Cell has missing pin: "<<portp->prettyName());
+			    AstPin* newp = new AstPin(nodep->fileline(),0,portp->name(),NULL);
+			    nodep->addPinsp(newp);
+			}
+		    }
+		}
+	    }
+	}
 	if (nodep->modp()) {
 	    nodep->iterateChildren(*this);
 	}
+	UINFO(4," Link Cell done: "<<nodep<<endl);
     }
 
     // Accelerate the recursion
@@ -277,16 +301,16 @@ private:
 	// Look at all modules, and store pointers to all module names
 	for (AstNodeModule* nextp,* nodep = v3Global.rootp()->modulesp(); nodep; nodep=nextp) {
 	    nextp = nodep->nextp()->castNodeModule();
-	    AstNode* foundp = m_mods.findIdUpward(nodep->name());
+	    AstNode* foundp = m_mods.rootp()->findIdFallback(nodep->name())->nodep();
 	    if (foundp && foundp != nodep) {
 		if (!(foundp->fileline()->warnIsOff(V3ErrorCode::MODDUP) || nodep->fileline()->warnIsOff(V3ErrorCode::MODDUP))) {
-		    nodep->v3warn(MODDUP,"Duplicate declaration of module: "<<nodep->prettyName());
-		    foundp->v3warn(MODDUP,"... Location of original declaration");
+		    nodep->v3warn(MODDUP,"Duplicate declaration of module: "<<nodep->prettyName()<<endl
+				  <<foundp->warnMore()<<"... Location of original declaration");
 		}
 		nodep->unlinkFrBack();
 		pushDeletep(nodep); nodep=NULL;
 	    } else if (!foundp) {
-		m_mods.insert(nodep->name(), nodep);
+		m_mods.rootp()->insert(nodep->name(), new VSymEnt(&m_mods, nodep));
 	    }
 	}
 	//if (debug()>=9) m_mods.dump(cout, "-syms: ");
@@ -294,8 +318,10 @@ private:
 
 public:
     // CONSTUCTORS
-    LinkCellsVisitor(AstNetlist* rootp, V3InFilter* filterp) {
+    LinkCellsVisitor(AstNetlist* rootp, V3InFilter* filterp, V3ParseSym* parseSymp)
+	: m_mods(rootp) {
 	m_filterp = filterp;
+	m_parseSymp = parseSymp;
 	m_modp = NULL;
 	m_libVertexp = NULL;
 	m_topVertexp = NULL;
@@ -307,7 +333,7 @@ public:
 //######################################################################
 // Link class functions
 
-void V3LinkCells::link(AstNetlist* rootp, V3InFilter* filterp) {
+void V3LinkCells::link(AstNetlist* rootp, V3InFilter* filterp, V3ParseSym* parseSymp) {
     UINFO(4,__FUNCTION__<<": "<<endl);
-    LinkCellsVisitor visitor (rootp, filterp);
+    LinkCellsVisitor visitor (rootp, filterp, parseSymp);
 }
