@@ -4,8 +4,6 @@
 //
 // Code available from: http://www.veripool.org/verilator
 //
-// AUTHORS: Wilson Snyder with Paul Wasson, Duane Gabli
-//
 //*************************************************************************
 //
 // Copyright 2003-2012 by Wilson Snyder.  This program is free software; you can
@@ -28,12 +26,13 @@
 #include "V3Ast.h"
 
 class V3InFilter;
+class V3ParseSym;
 
 //============================================================================
 
 class V3LinkCells {
 public:
-    static void link(AstNetlist* nodep, V3InFilter* filterp);
+    static void link(AstNetlist* nodep, V3InFilter* filterp, V3ParseSym* parseSymp);
 };
 
 #endif // Guard
