@@ -4,8 +4,6 @@
 //
 // Code available from: http://www.veripool.org/verilator
 //
-// AUTHORS: Wilson Snyder with Paul Wasson, Duane Gabli
-//
 //*************************************************************************
 //
 // Copyright 2003-2012 by Wilson Snyder.  This program is free software; you can
@@ -116,6 +114,7 @@ private:
 
     // VISITORS
     virtual void visit(AstNodeModule* nodep, AstNUser*) {
+	if (nodep->dead()) return;
 	m_modp = nodep;
 	m_repeatNum = 0;
 	nodep->iterateChildren(*this);
@@ -140,9 +139,7 @@ private:
    	string name = string("__Vrepeat")+cvtToStr(m_repeatNum++);
 	// Spec says value is integral, if negative is ignored
 	AstVar* varp = new AstVar(nodep->fileline(), AstVarType::BLOCKTEMP, name,
-				  VFlagBitPacked(), 32);
-	varp->numeric(AstNumeric::SIGNED);
-	varp->dtypep()->numeric(AstNumeric::SIGNED);
+				  nodep->findSigned32DType());
 	varp->usedLoopIdx(true);
 	m_modp->addStmtp(varp);
 	AstNode* initsp = new AstAssign(nodep->fileline(), new AstVarRef(nodep->fileline(), varp, true),
