@@ -4,8 +4,6 @@
 //
 // Code available from: http://www.veripool.org/verilator
 //
-// AUTHORS: Wilson Snyder with Paul Wasson, Duane Gabli
-//
 //*************************************************************************
 //
 // Copyright 2003-2012 by Wilson Snyder.  This program is free software; you can
@@ -46,7 +44,6 @@ private:
 
     // STATE
     bool	m_setRefLvalue;	// Set VarRefs to lvalues for pin assignments
-    AstInitial*	m_initialp;	// In an initial block
     AstNodeFTask* m_ftaskp;	// Function or task we're inside
 
     // METHODS
@@ -69,10 +66,6 @@ private:
 		    nodep->v3warn(ASSIGNIN,"Assigning to input variable: "<<nodep->prettyName());
 		}
 	    }
-	    if (nodep->lvalue() && nodep->varp()->isConst()
-		&& !m_initialp) {  // Too loose, but need to allow our generated first assignment
-		nodep->v3error("Assigning to const variable: "<<nodep->prettyName());
-	    }
 	}
 	nodep->iterateChildren(*this);
     }
@@ -228,11 +221,6 @@ private:
 	}
 	m_setRefLvalue = last_setRefLvalue;
     }
-    virtual void visit(AstInitial* nodep, AstNUser*) {
-	m_initialp = nodep;
-	nodep->iterateChildren(*this);
-	m_initialp = NULL;
-    }
     virtual void visit(AstNodeFTask* nodep, AstNUser*) {
 	m_ftaskp = nodep;
 	nodep->iterateChildren(*this);
@@ -270,7 +258,6 @@ public:
     LinkLValueVisitor(AstNode* nodep, bool start) {
 	m_setRefLvalue = start;
 	m_ftaskp = NULL;
-	m_initialp = NULL;
 	nodep->accept(*this);
     }
     virtual ~LinkLValueVisitor() {}
