@@ -4,8 +4,6 @@
 //
 // Code available from: http://www.veripool.org/verilator
 //
-// AUTHORS: Wilson Snyder with Paul Wasson, Duane Gabli
-//
 //*************************************************************************
 //
 // Copyright 2003-2012 by Wilson Snyder.  This program is free software; you can
@@ -124,7 +122,6 @@ void V3LinkLevel::wrapTop(AstNetlist* netlistp) {
 							varp, oldvarp->isOutput()));
 		// Skip length and width comp; we know it's a direct assignment
 		pinp->modVarp(oldvarp);
-		pinp->widthSignedFrom(oldvarp);
 		cellp->addPinsp(pinp);
 	    }
 	}
