@@ -4,8 +4,6 @@
 //
 // Code available from: http://www.veripool.org/verilator
 //
-// AUTHORS: Wilson Snyder with Paul Wasson, Duane Gabli
-//
 //*************************************************************************
 //
 // Copyright 2003-2012 by Wilson Snyder.  This program is free software; you can
@@ -68,9 +66,13 @@ private:
     }
 
     // VISITs
+    // TODO: Most of these visitors are here for historical reasons.
+    // TODO: ExpectDecriptor can move to data type resolution, and the rest
+    // TODO: could move to V3LinkParse to get them out of the way of elaboration
     virtual void visit(AstNodeModule* nodep, AstNUser*) {
 	// Module: Create sim table for entire module and iterate
 	UINFO(8,"MODULE "<<nodep<<endl);
+	if (nodep->dead()) return;
 	m_modp = nodep;
 	m_senitemCvtNum = 0;
 	nodep->iterateChildren(*this);
@@ -92,13 +94,6 @@ private:
 
     virtual void visit(AstVar* nodep, AstNUser*) {
 	nodep->iterateChildren(*this);
-	if (nodep->isIO() && !(nodep->dtypeSkipRefp()->castBasicDType() ||
-			       nodep->dtypeSkipRefp()->castArrayDType())) {
-	    nodep->v3error("Unsupported: Inputs and outputs must be simple data types");
-	}
-	if (nodep->dtypeSkipRefp()->castConstDType()) {
-	    nodep->isConst(true);
-	}
 	if (m_ftaskp) nodep->funcLocal(true);
 	if (nodep->isSigModPublic()) {
 	    nodep->sigModPublic(false);  // We're done with this attribute
@@ -205,10 +200,15 @@ private:
 	    // So we replicate it in another node
 	    // Note that V3Param knows not to replace AstVarRef's under AstAttrOf's
 	    AstNode* basefromp = AstArraySel::baseFromp(nodep);
-	    AstNodeVarRef* varrefp = basefromp->castNodeVarRef();  // Maybe varxref - so need to clone
-	    if (!varrefp) nodep->v3fatalSrc("Illegal bit select; no signal being extracted from");
-	    nodep->attrp(new AstAttrOf(nodep->fileline(), AstAttrType::VAR_BASE,
-				       varrefp->cloneTree(false)));
+	    if (AstNodeVarRef* varrefp = basefromp->castNodeVarRef()) {  // Maybe varxref - so need to clone
+		nodep->attrp(new AstAttrOf(nodep->fileline(), AstAttrType::VAR_BASE,
+					   varrefp->cloneTree(false)));
+	    } else if (AstMemberSel* fromp = nodep->fromp()->castMemberSel()) {
+		nodep->attrp(new AstAttrOf(nodep->fileline(), AstAttrType::MEMBER_BASE,
+					   fromp->cloneTree(false)));
+	    } else {
+		nodep->v3fatalSrc("Illegal bit select; no signal/member being extracted from");
+	    }
 	}
     }
 
@@ -267,10 +267,10 @@ private:
 		    break;
 		default:  // Most operators, just move to next argument
 		    if (!V3Number::displayedFmtLegal(ch)) {
-			nodep->v3error("Unknown $display format code: %"<<ch);
+			nodep->v3error("Unknown $display-like format code: %"<<ch);
 		    } else {
 			if (!argp) {
-			    nodep->v3error("Missing arguments for $display format");
+			    nodep->v3error("Missing arguments for $display-like format");
 			} else {
 			    argp = argp->nextp();
 			}
@@ -280,7 +280,7 @@ private:
 	    }
 	}
 	if (argp) {
-	    argp->v3error("Extra arguments for $display format");
+	    argp->v3error("Extra arguments for $display-like format");
 	}
     }
 
