@@ -4,8 +4,6 @@
 //
 // Code available from: http://www.veripool.org/verilator
 //
-// AUTHORS: Wilson Snyder with Paul Wasson, Duane Gabli
-//
 //*************************************************************************
 //
 // Copyright 2003-2012 by Wilson Snyder.  This program is free software; you can
@@ -401,7 +399,7 @@ bool V3Number::displayedFmtLegal(char format) {
 
 string V3Number::displayed(const string& vformat) const {
     string::const_iterator pos = vformat.begin();
-    UASSERT(pos != vformat.end() && pos[0]=='%', "display with non format argument "<<*this);
+    UASSERT(pos != vformat.end() && pos[0]=='%', "$display-like function with non format argument "<<*this);
     ++pos;
     string fmtsize;
     for (; pos != vformat.end() && (isdigit(pos[0]) || pos[0]=='.'); ++pos) {
@@ -444,7 +442,7 @@ string V3Number::displayed(const string& vformat) const {
 	return str;
     }
     case 'c': {
-	if (this->width()>8) m_fileline->v3error("$display of char format of > 8 bit value");
+	if (this->width()>8) m_fileline->v3error("$display-like format of char of > 8 bit value");
 	int v = bitsValue(0, 8);
 	str += (char)(v);
 	return str;
@@ -477,7 +475,7 @@ string V3Number::displayed(const string& vformat) const {
 	    fmtsize = cvtToStr(int(dchars));
 	}
 	if (width() > 64) {
-	    m_fileline->v3error("Unsupported: $display of dec format of > 64 bit results (use hex format instead)");
+	    m_fileline->v3error("Unsupported: $display-like format of decimal of > 64 bit results (use hex format instead)");
 	    return "ERR";
 	}
 	if (issigned) {
@@ -501,7 +499,7 @@ string V3Number::displayed(const string& vformat) const {
 	return tmp;
     }
     default:
-	m_fileline->v3fatalSrc("Unknown $display format code for number: %"<<pos[0]);
+	m_fileline->v3fatalSrc("Unknown $display-like format code for number: %"<<pos[0]);
 	return "ERR";
     }
 }
@@ -916,6 +914,18 @@ last:
     return setSingleBits(outc);
 }
 
+V3Number& V3Number::opLogIf (const V3Number& lhs, const V3Number& rhs) {
+    // i op j, 1 bit return, max(L(lhs),L(rhs)) calculation, careful need to
+    // X/Z extend. Use opLogNot and opLogOr to do this for us.
+    return opLogOr(opLogNot(lhs), rhs);
+}
+
+V3Number& V3Number::opLogIff (const V3Number& lhs, const V3Number& rhs) {
+    // i op j, 1 bit return, max(L(lhs),L(rhs)) calculation, careful need to
+    // X/Z extend. Use opLogNot and opLogXor to do this for us.
+    return opLogNot(opXor(lhs, rhs));
+}
+
 V3Number& V3Number::opEq (const V3Number& lhs, const V3Number& rhs) {
     // i op j, 1 bit return, max(L(lhs),L(rhs)) calculation, careful need to X/Z extend.
     char outc = 1;
