@@ -4,8 +4,6 @@
 //
 // Code available from: http://www.veripool.org/verilator
 //
-// AUTHORS: Wilson Snyder with Paul Wasson, Duane Gabli
-//
 //*************************************************************************
 //
 // Copyright 2003-2012 by Wilson Snyder.  This program is free software; you can
@@ -56,6 +54,7 @@ class ParamVisitor : public AstNVisitor {
 private:
     // NODE STATE
     //	 AstNodeModule::user5()	// bool	  True if processed
+    //	 AstGenFor::user5()	// bool	  True if processed
     //	 AstVar::user5()	// bool	  True if constant propagated
     //   AstVar::user4()	// int    Global parameter number (for naming new module)
     //				//        (0=not processed, 1=iterated, but no number, 65+ parameter numbered)
@@ -77,8 +76,8 @@ private:
     LongMap	m_longMap;	// Hash of very long names to unique identity number
     int		m_longId;
 
-    typedef deque<AstNodeModule*> ModDeque;
-    ModDeque	m_todoModps;	// Modules left to process
+    typedef multimap<int,AstNodeModule*> LevelModMap;
+    LevelModMap	m_todoModps;	// Modules left to process
 
     // METHODS
     static int debug() {
@@ -126,11 +125,13 @@ private:
     }
     void visitModules() {
 	// Loop on all modules left to process
-	// Hitting a cell adds to the END of this list, so since cells originally exist top->bottom
-	// we process in top->bottom order too.
+	// Hitting a cell adds to the appropriate leval of this level-sorted list,
+	// so since cells originally exist top->bottom we process in top->bottom order too.
 	while (!m_todoModps.empty()) {
-	    AstNodeModule* nodep = m_todoModps.front(); m_todoModps.pop_front();
-	    if (!nodep->user5Inc()) {  // Process once; note clone() must clear so we do it again
+	    LevelModMap::iterator it = m_todoModps.begin();
+	    AstNodeModule* nodep = it->second;
+	    m_todoModps.erase(it);
+	    if (!nodep->user5SetOnce()) {  // Process once; note clone() must clear so we do it again
 		UINFO(4," MOD   "<<nodep<<endl);
 		nodep->iterateChildren(*this);
 		// Note this may add to m_todoModps
@@ -144,9 +145,11 @@ private:
 	nodep->iterateChildren(*this);
     }
     virtual void visit(AstNodeModule* nodep, AstNUser*) {
-	if (nodep->level() <= 2) {  // Haven't added top yet, so level 2 is the top
+	if (nodep->dead()) {
+	    UINFO(4," MOD-dead.  "<<nodep<<endl);  // Marked by LinkDot
+	} else if (nodep->level() <= 2) {  // Haven't added top yet, so level 2 is the top
 	    // Add request to END of modules left to process
-	    m_todoModps.push_back(nodep);
+	    m_todoModps.insert(make_pair(nodep->level(),nodep));
 	    visitModules();
 	} else if (nodep->user5()) {
 	    UINFO(4," MOD-done   "<<nodep<<endl);  // Already did it
@@ -158,7 +161,7 @@ private:
 
     // Make sure all parameters are constantified
     virtual void visit(AstVar* nodep, AstNUser*) {
-	if (!nodep->user5Inc()) {  // Process once
+	if (!nodep->user5SetOnce()) {  // Process once
 	    nodep->iterateChildren(*this);
 	    if (nodep->isParam()) {
 		if (!nodep->hasSimpleInit()) { nodep->v3fatalSrc("Parameter without initial value"); }
@@ -189,8 +192,12 @@ private:
     virtual void visit(AstGenIf* nodep, AstNUser*) {
 	UINFO(9,"  GENIF "<<nodep<<endl);
 	nodep->condp()->iterateAndNext(*this);
-	V3Width::widthParamsEdit(nodep);  // Param typed widthing will NOT recurse the body
-	V3Const::constifyParamsEdit(nodep->condp());  // condp may change
+	// We suppress errors when widthing params since short-circuiting in
+	// the conditional evaluation may mean these error can never occur. We
+	// then make sure that short-circuiting is used by constifyParamsEdit.
+	V3Width::widthGenerateParamsEdit(nodep);  // Param typed widthing will
+						  // NOT recurse the body.
+	V3Const::constifyGenerateParamsEdit(nodep->condp()); // condp may change
 	if (AstConst* constp = nodep->condp()->castConst()) {
 	    AstNode* keepp = (constp->isZero()
 			      ? nodep->elsesp()
@@ -207,20 +214,49 @@ private:
 	    nodep->condp()->v3error("Generate If condition must evaluate to constant");
 	}
     }
+
+    //! Parameter subsitution for generated for loops.
+    //! @todo Unlike generated IF, we don't have to worry about short-circuiting the conditional
+    //!       expression, since this is currently restricted to simple comparisons. If we ever do
+    //!       move to more generic constant expressions, such code will be neede here.
+    virtual void visit(AstBegin* nodep, AstNUser*) {
+	if (nodep->genforp()) {
+	    AstGenFor* forp = nodep->genforp()->castGenFor();
+	    if (!forp) nodep->v3fatalSrc("Non-GENFOR under generate-for BEGIN");
+	    // We should have a GENFOR under here.  We will be replacing the begin,
+	    // so process here rather than at the generate to avoid iteration problems
+	    UINFO(9,"  BEGIN "<<nodep<<endl);
+	    UINFO(9,"  GENFOR "<<forp<<endl);
+	    V3Width::widthParamsEdit(forp);  // Param typed widthing will NOT recurse the body
+	    // Outer wrapper around generate used to hold genvar, and to insure genvar
+	    // doesn't conflict in V3LinkDot resolution with other genvars
+	    // Now though we need to change BEGIN("zzz",GENFOR(...)) to
+	    // a BEGIN("zzz__BRA__{loop#}__KET__")
+	    string beginName = nodep->name();
+	    // Leave the original Begin, as need a container for the (possible) GENVAR
+	    // Note V3Unroll will replace some AstVarRef's to the loop variable with constants
+	    V3Unroll::unrollGen(forp, beginName); forp=NULL;
+	    // Blocks were constructed under the special begin, move them up
+	    // Note forp is null, so grab statements again
+	    if (AstNode* stmtsp = nodep->genforp()) {
+		stmtsp->unlinkFrBackWithNext();
+		nodep->addNextHere(stmtsp);
+		// Note this clears nodep->genforp(), so begin is no longer special
+	    }
+	} else {
+	    nodep->iterateChildren(*this);
+	}
+    }
     virtual void visit(AstGenFor* nodep, AstNUser*) {
-	// We parse a very limited form of FOR, so we don't need to do a full
-	// simulation to unroll the loop
-	UINFO(9,"  GENFOR "<<nodep<<endl);
-	V3Width::widthParamsEdit(nodep);  // Param typed widthing will NOT recurse the body
-	// Note V3Unroll will replace some AstVarRef's to the loop variable with constants
-	V3Unroll::unrollGen(nodep); nodep=NULL;
+	nodep->v3fatalSrc("GENFOR should have been wrapped in BEGIN");
     }
     virtual void visit(AstGenCase* nodep, AstNUser*) {
 	UINFO(9,"  GENCASE "<<nodep<<endl);
 	AstNode* keepp = NULL;
 	nodep->exprp()->iterateAndNext(*this);
 	V3Case::caseLint(nodep);
-	V3Width::widthParamsEdit(nodep);  // Param typed widthing will NOT recurse the body
+	V3Width::widthParamsEdit(nodep);  // Param typed widthing will NOT recurse the body,
+					  // don't trigger errors yet.
 	V3Const::constifyParamsEdit(nodep->exprp());  // exprp may change
 	AstConst* exprp = nodep->exprp()->castConst();
 	// Constify
@@ -301,6 +337,7 @@ void ParamVisitor::visit(AstCell* nodep, AstNUser*) {
 	if (debug()>8) nodep->paramsp()->dumpTreeAndNext(cout,"-cellparams:\t");
 	for (AstPin* pinp = nodep->paramsp(); pinp; pinp=pinp->nextp()->castPin()) {
 	    if (!pinp) nodep->v3fatalSrc("Non pin under cell params\n");
+	    if (!pinp->exprp()) continue; // No-connect
 	    AstVar* modvarp = pinp->modVarp();
 	    if (!modvarp) {
 		pinp->v3error("Parameter not found in sub-module: Param "<<pinp->name()<<" of "<<nodep->prettyName());
@@ -312,7 +349,7 @@ void ParamVisitor::visit(AstCell* nodep, AstNUser*) {
 		if (!constp) {
 		    //if (debug()) pinp->dumpTree(cout,"error:");
 		    pinp->v3error("Can't convert defparam value to constant: Param "<<pinp->name()<<" of "<<nodep->prettyName());
-		    pinp->exprp()->replaceWith(new AstConst(pinp->fileline(), V3Number(pinp->fileline(), pinp->width(), 0)));
+		    pinp->exprp()->replaceWith(new AstConst(pinp->fileline(), V3Number(pinp->fileline(), modvarp->width(), 0)));
 		} else if (origconstp && constp->sameTree(origconstp)) {
 		    // Setting parameter to its default value.  Just ignore it.
 		    // This prevents making additional modules, and makes coverage more
@@ -353,7 +390,7 @@ void ParamVisitor::visit(AstCell* nodep, AstNUser*) {
 		// However links outside the module (like on the upper cells) will not.
 		modp = nodep->modp()->cloneTree(false);
 		modp->name(newname);
-		modp->user5(0); // We need to re-recurse this module once changed
+		modp->user5(false); // We need to re-recurse this module once changed
 		nodep->modp()->addNextHere(modp);  // Keep tree sorted by cell occurrences
 
 		m_modNameMap.insert(make_pair(modp->name(), ModInfo(modp)));
@@ -380,7 +417,7 @@ void ParamVisitor::visit(AstCell* nodep, AstNUser*) {
 		// Assign parameters to the constants specified
 		for (AstPin* pinp = nodep->paramsp(); pinp; pinp=pinp->nextp()->castPin()) {
 		    AstVar* modvarp = pinp->modVarp();
-		    if (modvarp) {
+		    if (modvarp && pinp->exprp()) {
 			AstConst* constp = pinp->exprp()->castConst();
 			// Remove any existing parameter
 			if (modvarp->valuep()) modvarp->valuep()->unlinkFrBack()->deleteTree();
@@ -408,7 +445,7 @@ void ParamVisitor::visit(AstCell* nodep, AstNUser*) {
     }
 
     // Now remember to process the child module at the end of the module
-    m_todoModps.push_back(nodep->modp());
+    m_todoModps.insert(make_pair(nodep->modp()->level(),nodep->modp()));
 }
 
 //######################################################################
