@@ -4,8 +4,6 @@
 //
 // Code available from: http://www.veripool.org/verilator
 //
-// AUTHORS: Wilson Snyder with Paul Wasson, Duane Gabli
-//
 //*************************************************************************
 //
 // Copyright 2009-2012 by Wilson Snyder.  This program is free software; you can
@@ -27,7 +25,7 @@
 #include "V3Error.h"
 #include "V3Global.h"
 #include "V3Parse.h"
-#include "V3SymTable.h"
+#include "V3ParseSym.h"
 #include <deque>
 
 class V3Lexer;
@@ -55,7 +53,7 @@ struct V3ParseBisonYYSType {
 	double		cdouble;
 	bool		cbool;
 	V3UniqState	uniqstate;
-	AstSignedState	signstate;
+	VSignedState	signstate;
 	V3ImportProperty iprop;
 	V3ErrorCode::en	errcodeen;
 
@@ -65,8 +63,11 @@ struct V3ParseBisonYYSType {
 	AstBegin*	beginp;
 	AstCase*	casep;
 	AstCaseItem*	caseitemp;
+	AstCell*	cellp;
 	AstConst*	constp;
+	AstMemberDType*	memberp;
 	AstNodeModule*	modulep;
+	AstNodeClassDType* classp;
 	AstNodeDType*	dtypep;
 	AstNodeFTask*	ftaskp;
 	AstNodeFTaskRef* ftaskrefp;
@@ -74,6 +75,7 @@ struct V3ParseBisonYYSType {
 	AstNodeVarRef*	varnodep;
 	AstPackage*	packagep;
 	AstParseRef*	parserefp;
+	AstPatMember*	patmemberp;
 	AstPin*		pinp;
 	AstRange*	rangep;
 	AstSenTree*	sentreep;
@@ -84,138 +86,18 @@ struct V3ParseBisonYYSType {
 
 #define YYSTYPE V3ParseBisonYYSType
 
-//######################################################################
-// Symbol table for parsing
-
-class V3ParseSym {
-    // TYPES
-    typedef vector<V3SymTable*>	SymStack;
-
-private:
-    // MEMBERS
-    static int	s_anonNum;		// Number of next anonymous object
-    V3SymTable*	m_symTableNextId;	// Symbol table for next lexer lookup
-    V3SymTable*	m_symCurrentp;		// Active symbol table for additions/lookups
-    V3SymTable*	m_symRootp;		// Root symbol table
-    SymStack	m_sympStack;		// Stack of nodes with symbol tables
-    SymStack	m_symsp;		// All symbol tables, to cleanup
-
-private:
-    // METHODS
-    static V3SymTable* getTable(AstNode* nodep) {
-	if (!nodep->user4p()) nodep->v3fatalSrc("Current symtable not found");
-	return nodep->user4p()->castSymTable();
-    }
-
-public:
-    V3SymTable* nextId() const { return m_symTableNextId; }
-    V3SymTable* symCurrentp() const { return m_symCurrentp; }
-    V3SymTable*	symRootp() const { return m_symRootp; }
-
-    V3SymTable* findNewTable(AstNode* nodep, V3SymTable* parentp) {
-	if (!nodep->user4p()) {
-	    V3SymTable* symsp = new V3SymTable(nodep, parentp);
-	    nodep->user4p(symsp);
-	    m_symsp.push_back(symsp);
-	}
-	return getTable(nodep);
-    }
-    void nextId(AstNode* entp) {
-	if (entp) {
-	    UINFO(9,"symTableNextId under "<<entp<<"-"<<entp->type().ascii()<<endl);
-	    m_symTableNextId = getTable(entp);
-	}
-	else {
-	    UINFO(9,"symTableNextId under NULL"<<endl);
-	    m_symTableNextId = NULL;
-	}
-    }
-    void reinsert(AstNode* nodep, V3SymTable* parentp=NULL) {
-	reinsert(nodep, parentp, nodep->name());
-    }
-    void reinsert(AstNode* nodep, V3SymTable* parentp, string name) {
-	if (!parentp) parentp = symCurrentp();
-	if (name == "") {  // New name with space in name so can't collide with users
-	    name = string(" anon") + nodep->type().ascii() + cvtToStr(++s_anonNum);
-	}
-	parentp->reinsert(name,nodep);
-    }
-    void pushNew(AstNode* nodep) { pushNewUnder(nodep, NULL); }
-    void pushNewUnder(AstNode* nodep, V3SymTable* parentp) {
-	if (!parentp) parentp = symCurrentp();
-	V3SymTable* symp = findNewTable(nodep, parentp);  // Will set user4p, which is how we connect table to node
-	reinsert(nodep, parentp);
-	pushScope(symp);
-    }
-    void pushScope(V3SymTable* symp) {
-	m_sympStack.push_back(symp);
-	m_symCurrentp = symp;
-    }
-    void popScope(AstNode* nodep) {
-	if (symCurrentp()->ownerp() != nodep) {
-	    if (debug()) { showUpward(); dump(cout,"-mism: "); }
-	    nodep->v3fatalSrc("Symbols suggest ending "<<symCurrentp()->ownerp()->prettyTypeName()
-			      <<" but parser thinks ending "<<nodep->prettyTypeName());
-	    return;
-	}
-	m_sympStack.pop_back();
-	if (m_sympStack.empty()) { nodep->v3fatalSrc("symbol stack underflow"); return; }
-	m_symCurrentp = m_sympStack.back();
-    }
-    void showUpward () {
-	UINFO(1,"ParseSym Stack:\n");
-	for (SymStack::reverse_iterator it=m_sympStack.rbegin(); it!=m_sympStack.rend(); ++it) {
-	    V3SymTable* symp = *it;
-	    UINFO(1,"\t"<<symp->ownerp()<<endl);
-	}
-	UINFO(1,"ParseSym Current: "<<symCurrentp()->ownerp()<<endl);
-    }
-    void dump(ostream& os, const string& indent="") {
-	os<<"ParseSym Dump:\n";
-	m_sympStack[0]->dump(os, indent, true);
-    }
-    AstNode* findEntUpward (const string& name) {
-	// Lookup the given string as an identifier, return type of the id, scanning upward
-	return symCurrentp()->findIdUpward(name);
-    }
-    void import(AstNode* packagep, const string& id_or_star) {
-	// Import from package::id_or_star to this
-	V3SymTable* symp = getTable(packagep);
-	if (!symp) {  // Internal problem, because we earlier found pkg to label it an ID__aPACKAGE
-	    packagep->v3fatalSrc("Import package not found");
-	    return;
-	}
-	// Walk old sym table and reinsert into current table
-	// We let V3Link report the error instead of us
-	symCurrentp()->import(symp, id_or_star);
-    }
-public:
-    // CREATORS
-    V3ParseSym(AstNetlist* rootp) {
-	s_anonNum = 0;		// Number of next anonymous object
-	pushScope(findNewTable(rootp, NULL));
-	m_symTableNextId = NULL;
-	m_symCurrentp = symCurrentp();
-	m_symRootp = symCurrentp();
-    }
-    ~V3ParseSym() {
-	for (SymStack::iterator it = m_symsp.begin(); it != m_symsp.end(); ++it) {
-	    delete (*it);
-	}
-    }
-};
-
 //######################################################################
 
 class V3ParseImp {
     // MEMBERS
     AstNetlist* 	m_rootp;	// Root of the design
     V3InFilter*		m_filterp;	// Reading filter
+    V3ParseSym*		m_symp;		// Symbol table
+
     V3Lexer*		m_lexerp;	// Current FlexLexer
     static V3ParseImp*	s_parsep;	// Current THIS, bison() isn't class based
     FileLine*		m_fileline;	// Filename/linenumber currently active
 
-    V3ParseSym	m_sym;			// Symbol table
     bool	m_inCellDefine;		// Inside a `celldefine
     bool	m_inLibrary;		// Currently reading a library vs. regular file
     int		m_inBeginKwd;		// Inside a `begin_keywords
@@ -312,14 +194,14 @@ public:
     size_t flexPpInputToLex(char* buf, size_t max_size) { return ppInputToLex(buf,max_size); }
 
     //==== Symbol tables
-    V3ParseSym* symp() { return &m_sym; }
+    V3ParseSym* symp() { return m_symp; }
 
 public:
     // CREATORS
-    V3ParseImp(AstNetlist* rootp, V3InFilter* filterp)
-	: m_filterp(filterp), m_sym(rootp) {
+    V3ParseImp(AstNetlist* rootp, V3InFilter* filterp, V3ParseSym* parserSymp)
+	: m_rootp(rootp), m_filterp(filterp), m_symp(parserSymp) {
 	m_fileline = NULL;
-	m_rootp = rootp; m_lexerp = NULL;
+	m_lexerp = NULL;
 	m_inCellDefine = false;
 	m_inLibrary = false;
 	m_inBeginKwd = 0;
