@@ -4,8 +4,6 @@
 //
 // Code available from: http://www.veripool.org/verilator
 //
-// AUTHORS: Wilson Snyder with Paul Wasson, Duane Gabli
-//
 //*************************************************************************
 //
 // Copyright 2003-2012 by Wilson Snyder.  This program is free software; you can
@@ -102,7 +100,7 @@ private:
 
     // Checking METHODS
 public:
-    /// Call other-this function on all new var references
+    /// Call other-this function on all new *non-constant* var references
     virtual void varRefCb(AstVarRef* nodep) {}
 
     void clearOptimizable(AstNode* nodep/*null ok*/, const string& why) {
@@ -272,7 +270,15 @@ private:
 	    if (!(vscp->user1() & VU_RV)) {
 		if (!m_params && (vscp->user1() & VU_LV)) clearOptimizable(nodep,"Var write & read");
 		vscp->user1( vscp->user1() | VU_RV);
-		if (m_checkOnly) varRefCb (nodep);
+		bool isConst = nodep->varp()->isParam();
+		AstConst* constp = (isConst ? nodep->varp()->valuep()->castConst() : NULL);
+		if (isConst && constp) { // Propagate PARAM constants for constant function analysis
+		    if (!m_checkOnly && optimizable()) {
+			newNumber(vscp)->opAssign(constp->num());
+		    }
+		} else {
+		    if (m_checkOnly) varRefCb (nodep);
+		}
 	    }
 	}
 	if (!m_checkOnly && optimizable()) { // simulating
