@@ -4,8 +4,6 @@
 //
 // Code available from: http://www.veripool.org/verilator
 //
-// AUTHORS: Wilson Snyder with Paul Wasson, Duane Gabli
-//
 //*************************************************************************
 //
 // Copyright 2003-2012 by Wilson Snyder.  This program is free software; you can
@@ -280,6 +278,9 @@ class SliceVisitor : public AstNVisitor {
 	    }
 	    UINFO(9,"    ArraySel-child: "<<topp<<endl);
 	    AstArraySel* newp = new AstArraySel(nodep->fileline(), topp, new AstConst(nodep->fileline(),lsb));
+	    if (!newp->dtypep()) {
+		newp->v3fatalSrc("ArraySel dtyping failed when resolving slice");  // see ArraySel constructor
+	    }
 	    newp->user1p(refp);
 	    newp->start(lsb);
 	    newp->length(msb - lsb + 1);
