@@ -4,8 +4,6 @@
 //
 // Code available from: http://www.veripool.org/verilator
 //
-// AUTHORS: Wilson Snyder with Paul Wasson, Duane Gabli
-//
 //*************************************************************************
 //
 // Copyright 2005-2012 by Wilson Snyder.  This program is free software; you can
@@ -93,7 +91,7 @@ private:
     virtual void visit(AstVar* nodep, AstNUser*) {
 	allNodes(nodep);
 	nodep->iterateChildren(*this);
-	if (m_counting) {
+	if (m_counting && nodep->dtypep()) {
 	    if (nodep->isUsedClock()) ++m_statVarClock;
 	    if (nodep->dtypeSkipRefp()->castArrayDType()) ++m_statVarArray;
 	    else m_statVarBytes += nodep->dtypeSkipRefp()->widthTotalBytes();
