@@ -4,8 +4,6 @@
 //
 // Code available from: http://www.veripool.org/verilator
 //
-// AUTHORS: Wilson Snyder with Paul Wasson, Duane Gabli
-//
 //*************************************************************************
 //
 // Copyright 2003-2012 by Wilson Snyder.  This program is free software; you can
@@ -192,8 +190,10 @@ private:
 	FileLine* fl = nodep->fileline();
 	AstNodeDType* dtypep
 	    = new AstArrayDType (fl,
-				 new AstBasicDType(fl, VFlagBitPacked(), m_outVarps.size()),
-				 new AstRange (fl, VL_MASK_I(m_inWidth), 0));
+				 nodep->findBitDType(m_outVarps.size(),
+						     m_outVarps.size(), AstNumeric::UNSIGNED),
+				 new AstRange (fl, VL_MASK_I(m_inWidth), 0), false);
+	v3Global.rootp()->typeTablep()->addTypesp(dtypep);
 	AstVar* chgVarp
 	    = new AstVar (fl, AstVarType::MODULETEMP,
 			  "__Vtablechg" + cvtToStr(m_modTables),
@@ -237,10 +237,9 @@ private:
 	    AstVar* outvarp = outvscp->varp();
 	    FileLine* fl = nodep->fileline();
 	    AstNodeDType* dtypep
-		= new AstArrayDType (fl,
-				     // FUTURE: If support more types, below can use outvarp->dtype()
-				     new AstBasicDType(fl, VFlagLogicPacked(), outvarp->width()),
-				     new AstRange (fl, VL_MASK_I(m_inWidth), 0));
+		= new AstArrayDType (fl, outvarp->dtypep(),
+				     new AstRange (fl, VL_MASK_I(m_inWidth), 0), false);
+	    v3Global.rootp()->typeTablep()->addTypesp(dtypep);
 	    AstVar* tablevarp
 		= new AstVar (fl, AstVarType::MODULETEMP,
 			      "__Vtable" + cvtToStr(m_modTables) +"_"+outvarp->name(),
