@@ -4,8 +4,6 @@
 //
 // Code available from: http://www.veripool.org/verilator
 //
-// AUTHORS: Wilson Snyder with Paul Wasson, Duane Gabli
-//
 //*************************************************************************
 //
 // Copyright 2003-2012 by Wilson Snyder.  This program is free software; you can
@@ -144,8 +142,8 @@ public:
     }
     void checkPurity(AstNodeFTask* nodep, TaskBaseVertex* vxp) {
 	if (!vxp->pure()) {
-	    nodep->v3warn(IMPURE,"Unsupported: External variable referenced by non-inlined function/task: "<<nodep->prettyName());
-	    vxp->impureNode()->v3warn(IMPURE,"... Location of the external reference: "<<vxp->impureNode()->prettyName());
+	    nodep->v3warn(IMPURE,"Unsupported: External variable referenced by non-inlined function/task: "<<nodep->prettyName()<<endl
+			  <<vxp->impureNode()->warnMore()<<"... Location of the external reference: "<<vxp->impureNode()->prettyName());
 	}
 	// And, we need to check all tasks this task calls
 	for (V3GraphEdge* edgep = vxp->outBeginp(); edgep; edgep=edgep->outNextp()) {
@@ -336,7 +334,7 @@ private:
     }
     AstVarScope* createInputVar(AstCFunc* funcp, const string& name, AstBasicDTypeKwd kwd) {
 	AstVar* newvarp = new AstVar (funcp->fileline(), AstVarType::BLOCKTEMP, name,
-				      new AstBasicDType(funcp->fileline(), kwd));
+				      funcp->findBasicDType(kwd));
 	newvarp->funcLocal(true);
 	newvarp->combineType(AstVarType::INPUT);
 	funcp->addArgsp(newvarp);
@@ -668,7 +666,7 @@ private:
 	    // We could use 64-bits of a MD5/SHA hash rather than a string here,
 	    // but the compare is only done on first call then memoized, so it's not worth optimizing.
 	    string stmt;
-	    stmt += "static int __Vfuncnum = -1;\n";
+	    stmt += "static int __Vfuncnum = -1;\n";  // Static doesn't need save-restore as if below will re-fill proper value
 	    // First time init (faster than what the compiler does if we did a singleton
 	    stmt += "if (VL_UNLIKELY(__Vfuncnum==-1)) { __Vfuncnum = Verilated::exportFuncNum(\""+nodep->cname()+"\"); }\n";
 	    // If the find fails, it will throw an error
@@ -914,9 +912,9 @@ private:
 		m_dpiNames.insert(make_pair(nodep->cname(), make_pair(dpip, dpiproto)));
 	    }
 	    else if (iter->second.second != dpiproto) {
-		nodep->v3error("Duplicate declaration of DPI function with different formal arguments: "<<nodep->prettyName());
-		nodep->v3error("... New prototype:      "<<dpiproto);
-		iter->second.first->v3error("... Original prototype: "<<iter->second.second);
+		nodep->v3error("Duplicate declaration of DPI function with different formal arguments: "<<nodep->prettyName()<<endl
+			       <<nodep->warnMore()<<"... New prototype:      "<<dpiproto<<endl
+			       <<iter->second.first->warnMore()<<"... Original prototype: "<<iter->second.second);
 	    }
 	}
 
@@ -932,8 +930,8 @@ private:
 		    if (dpip) {
 			dpip->addArgsp(portp->cloneTree(false));
 			if (!portp->basicp() || portp->basicp()->keyword().isDpiUnsupported()) {
-			    portp->v3error("Unsupported: DPI argument of type "<<portp->basicp()->prettyTypeName());
-			    portp->v3error("... For best portability, use bit, byte, int, or longint");
+			    portp->v3error("Unsupported: DPI argument of type "<<portp->basicp()->prettyTypeName()<<endl
+					   <<portp->warnMore()<<"... For best portability, use bit, byte, int, or longint");
 			}
 		    }
 		} else {
@@ -1075,7 +1073,7 @@ private:
 	AstNode* prevInsStmtp = m_insStmtp;
 	m_insMode = IM_BEFORE;
 	m_insStmtp = nodep->stmtsp();  // Might be null if no statements, but we won't use it
-	if (!nodep->user1Inc()) {  // Just one creation needed per function
+	if (!nodep->user1SetOnce()) {  // Just one creation needed per function
 	    // Expand functions in it
 	    int modes = 0;
 	    if (nodep->dpiImport()) modes++;
