@@ -4,8 +4,6 @@
 //
 // Code available from: http://www.veripool.org/verilator
 //
-// AUTHORS: Wilson Snyder with Paul Wasson, Duane Gabli
-//
 //*************************************************************************
 //
 // Copyright 2003-2012 by Wilson Snyder.  This program is free software; you can
@@ -32,7 +30,6 @@
 class V3Tristate {
 public:
     static void tristateAll(AstNetlist* nodep);
-    static void inoutAll(AstNetlist* nodep);
 };
 
 #endif // Guard
