@@ -4,8 +4,6 @@
 //
 // Code available from: http://www.veripool.org/verilator
 //
-// AUTHORS: Wilson Snyder with Paul Wasson, Duane Gabli
-//
 //*************************************************************************
 //
 // Copyright 2004-2012 by Wilson Snyder.  This program is free software; you can
@@ -38,6 +36,7 @@
 #include <vector>
 
 #include "V3Global.h"
+#include "V3String.h"
 #include "V3Undriven.h"
 #include "V3Ast.h"
 
@@ -137,7 +136,7 @@ public:
     bool unusedMatch(AstVar* nodep) {
 	const char* regexpp = v3Global.opt.unusedRegexp().c_str();
 	if (!regexpp || !*regexpp) return false;
-	return V3Options::wildmatch(nodep->prettyName().c_str(), regexpp);
+	return VString::wildmatch(nodep->prettyName().c_str(), regexpp);
     }
     void reportViolations() {
 	// Combine bits into overall state
@@ -276,8 +275,9 @@ private:
     virtual void visit(AstVarRef* nodep, AstNUser*) {
 	// Any variable
 	UndrivenVarEntry* entryp = getEntryp (nodep->varp());
+	bool fdrv = nodep->lvalue() && nodep->varp()->attrFileDescr();  // FD's are also being read from
 	if (m_markBoth || nodep->lvalue()) entryp->drivenWhole();
-	if (m_markBoth || !nodep->lvalue()) entryp->usedWhole();
+	if (m_markBoth || !nodep->lvalue() || fdrv) entryp->usedWhole();
     }
 
     // Don't know what black boxed calls do, assume in+out
