@@ -4,8 +4,6 @@
 //
 // Code available from: http://www.veripool.org/verilator
 //
-// AUTHORS: Wilson Snyder with Paul Wasson, Duane Gabli
-//
 //*************************************************************************
 //
 // Copyright 2003-2012 by Wilson Snyder.  This program is free software; you can
@@ -53,11 +51,12 @@ private:
     AstVarScope*	m_forVscp;		// Iterator variable scope (NULL for generate pass)
     AstConst*		m_varValuep;		// Current value of loop
     AstNode*		m_ignoreIncp;		// Increment node to ignore
+    AstAttrOf*		m_attrp;		// Current attribute
     bool		m_varModeCheck;		// Just checking RHS assignments
     bool		m_varModeReplace;	// Replacing varrefs
     bool		m_varAssignHit;		// Assign var hit
-    bool		m_inBegin;		// Inside a begin/end loop
     bool		m_generate;		// Expand single generate For loop
+    string		m_beginName;		// What name to give begin iterations
     V3Double0		m_statLoops;		// Statistic tracking
     V3Double0		m_statIters;		// Statistic tracking
 
@@ -285,7 +284,12 @@ private:
 			m_varModeReplace = true;
 			tempp->stmtsp()->iterateAndNext(*this);
 			m_varModeReplace = false;
-			tempp->stmtsp()->unlinkFrBackWithNext(); tempp->deleteTree(); tempp=NULL;
+			oneloopp = tempp->stmtsp()->unlinkFrBackWithNext(); tempp->deleteTree(); tempp=NULL;
+		    }
+		    if (m_generate) {
+			string index = AstNode::encodeNumber(m_varValuep->toSInt());
+			string nname = m_beginName + "__BRA__" + index + "__KET__";
+			oneloopp = new AstBegin(oneloopp->fileline(),nname,oneloopp,true);
 		    }
 
 		    if (newbodysp) newbodysp->addNext(oneloopp);
@@ -312,6 +316,7 @@ private:
 	if (bodysp) { pushDeletep(bodysp); bodysp=NULL; }
 	if (precondsp) { pushDeletep(precondsp); precondsp=NULL; }
 	if (initp) { pushDeletep(initp); initp=NULL; }
+	if (incp && !incp->backp()) { pushDeletep(incp); incp=NULL; }
 	if (debug()>=9) newbodysp->dumpTree(cout,"-  _new: ");
     }
 
@@ -379,28 +384,11 @@ private:
 	    nodep->v3error("V3Begin should have removed standard FORs");
 	}
     }
-
-    virtual void visit(AstBegin* nodep, AstNUser*) {
-	// Naming inside loop body; must have been a generate for.
-	// We need to only rename the 'upper' begin,
-	// anything lower will be renamed "uppernewname.lowerbegin"
-	bool lastBegin = m_inBegin;
-	m_inBegin = true;
-	nodep->stmtsp()->iterateAndNext(*this);
-	m_inBegin = lastBegin;
-	nodep->flatsp()->iterateAndNext(*this);
-
-	if (m_varModeReplace && !m_inBegin // no upper begin, excluding this one
-	    ) {
-	    // Rename it, as otherwise we may get a conflict
-	    // V3Begin sees these DOTs and makes CellInlines for us.
-	    string index = AstNode::encodeNumber(m_varValuep->toSInt());
-	    string nname = (string)"genfor"+index+"__DOT__"+nodep->name();
-	    // Verilog seems to drop the for loop name and tack on [#]
-	    nname = nodep->name() + "__BRA__" + index + "__KET__";
-	    //UINFO(8,"   Rename begin "<<nname<<" "<<nodep<<endl);
-	    nodep->name(nname);
-	}
+    virtual void visit(AstAttrOf* nodep, AstNUser*) {
+	AstAttrOf* oldAttr = m_attrp;
+	m_attrp = nodep;
+	nodep->iterateChildren(*this);
+	m_attrp = oldAttr;
     }
 
     virtual void visit(AstVarRef* nodep, AstNUser*) {
@@ -415,7 +403,7 @@ private:
 	    && nodep->varp() == m_forVarp
 	    && nodep->varScopep() == m_forVscp
 	    && !nodep->lvalue()
-	    && !nodep->backp()->castAttrOf()) {  // Most likely under a select
+	    && !m_attrp) {  // Most likely under a select
 	    AstNode* newconstp = m_varValuep->cloneTree(false);
 	    nodep->replaceWith(newconstp);
 	    pushDeletep(nodep);
@@ -434,14 +422,15 @@ private:
 
 public:
     // CONSTUCTORS
-    UnrollVisitor(AstNode* nodep, bool generate) {
+    UnrollVisitor(AstNode* nodep, bool generate, string beginName) {
 	m_forVarp = NULL;
 	m_forVscp = NULL;
 	m_ignoreIncp = NULL;
 	m_varModeCheck = false;
 	m_varModeReplace = false;
-	m_inBegin = false;
 	m_generate = generate;
+	m_beginName = beginName;
+	m_attrp = NULL;
 	//
 	nodep->accept(*this);
     }
@@ -456,10 +445,10 @@ public:
 
 void V3Unroll::unrollAll(AstNetlist* nodep) {
     UINFO(2,__FUNCTION__<<": "<<endl);
-    UnrollVisitor visitor (nodep, false);
+    UnrollVisitor visitor (nodep, false, "");
 }
 
-void V3Unroll::unrollGen(AstNodeFor* nodep) {
+void V3Unroll::unrollGen(AstNodeFor* nodep, string beginName) {
     UINFO(2,__FUNCTION__<<": "<<endl);
-    UnrollVisitor visitor (nodep, true);
+    UnrollVisitor visitor (nodep, true, beginName);
 }
