@@ -4,8 +4,6 @@
 //
 // Code available from: http://www.veripool.org/verilator
 //
-// AUTHORS: Wilson Snyder with Paul Wasson, Duane Gabli
-//
 //*************************************************************************
 //
 // Copyright 2003-2012 by Wilson Snyder.  This program is free software; you can
@@ -77,21 +75,28 @@ private:
 
     AstNodeDType* dtypeForExtractp(AstNode* nodep, AstNode* basefromp, int dimension, bool rangedSelect) {
 	// Perform error checks on the node
-	AstVar* varp = varFromBasefrom(basefromp);
-	//UINFO(9,"SCD\n"); if (debug()>=9) nodep->backp()->dumpTree(cout,"-selcheck: ");
-	AstNodeDType* ddtypep = varp->dtypep()->dtypeDimensionp(dimension);
+	AstNode* errp;
+	AstNodeDType* bfdtypep = baseDTypeFrom(basefromp, errp/*ref*/);
+	//UINFO(9,"SCD\n"); if (debug()>=9) bfdtypep->dumpTree(cout,"-dtfexvar: ");
+	// This may be called on dotted variables before we've completed widthing of the dotted var.
+	AstNodeDType* ddtypep = bfdtypep;
+	ddtypep = ddtypep->dtypeDimensionp(dimension);
+	if (debug()>=9 &&ddtypep) ddtypep->dumpTree(cout,"-ddtypep: ");
 	if (AstArrayDType* adtypep = ddtypep->castArrayDType()) {
 	    return adtypep;
 	}
+	else if (AstNodeClassDType* adtypep = ddtypep->castNodeClassDType()) {
+	    return adtypep;
+	}
 	else if (AstBasicDType* adtypep = ddtypep->castBasicDType()) {
 	    if (!adtypep->isRanged()) {
-		nodep->v3error("Illegal bit select; variable does not have a bit range, or bad dimension: "<<varp->prettyName());
+		nodep->v3error("Illegal bit select; variable does not have a bit range, or bad dimension: "<<errp->prettyName());
 		return NULL;
 	    }
 	    return adtypep;
 	}
 	else {
-	    nodep->v3error("Illegal bit or array select; variable already selected, or bad dimension: "<<varp->prettyName());
+	    nodep->v3error("Illegal bit or array select; variable already selected, or bad dimension: "<<errp->prettyName());
 	}
 	return NULL;
     }
@@ -112,13 +117,13 @@ private:
 	    AstNode* newp = new AstSub(lhsp->fileline(), lhsp,
 				       new AstConst(lhsp->fileline(), AstConst::Unsized32(), rhs));
 	    // We must make sure sub gets sign of original value, not from the constant
-	    newp->numericFrom(lhsp);
+	    newp->dtypeFrom(lhsp);
 	    return newp;
 	} else {  // rhs < 0;
 	    AstNode* newp = new AstAdd(lhsp->fileline(), lhsp,
 				       new AstConst(lhsp->fileline(), AstConst::Unsized32(), -rhs));
 	    // We must make sure sub gets sign of original value, not from the constant
-	    newp->numericFrom(lhsp);
+	    newp->dtypeFrom(lhsp);
 	    return newp;
 	}
     }
@@ -128,38 +133,51 @@ private:
 	AstNode* newp = new AstSub(rhsp->fileline(),
 				   new AstConst(rhsp->fileline(), AstConst::Unsized32(), lhs),
 				   rhsp);
-	newp->widthSignedFrom(rhsp);  // Important as AstSub default is lhs's sign
+	newp->dtypeFrom(rhsp);  // Important as AstSub default is lhs's sign
 	return newp;
     }
 
-    AstVar* varFromBasefrom(AstNode* basefromp) {
-	AstNodeVarRef* varrefp = basefromp->castNodeVarRef();
-	if (!varrefp) basefromp->v3fatalSrc("Bit/array selection of non-variable");
-	AstVar* varp = varrefp->varp();	if (!varp) varrefp->v3fatalSrc("Signal not linked");
-	return varp;
+    AstNodeDType* baseDTypeFrom(AstNode* basefromp, AstNode*& errpr) {
+	errpr = basefromp;
+	if (AstNodeVarRef* varrefp = basefromp->castNodeVarRef()) {
+	    AstVar* varp = varrefp->varp(); if (!varp) { varrefp->v3fatalSrc("Signal not linked"); return NULL; }
+	    errpr = varp;
+	    AstNodeDType* bfdtypep = varp->subDTypep();
+	    if (!bfdtypep) basefromp->v3fatalSrc("No datatype found for variable in select");
+	    return bfdtypep;
+	} else if (AstMemberSel* selp = basefromp->castMemberSel()) {
+	    errpr = selp;
+	    AstNodeDType* bfdtypep = selp->dtypep();
+	    if (!bfdtypep) basefromp->v3fatalSrc("No datatype found for variable in select");
+	    return bfdtypep;
+	} else {
+	    basefromp->v3fatalSrc("Bit/array selection of non-variable");
+	}
+	return NULL;
     }
 
     AstNode* newSubLsbOf(AstNode* underp, AstNode* basefromp) {
 	// Account for a variable's LSB in bit selections
 	// Will likely become SUB(underp, lsb_of_signal)
 	// Don't report WIDTH warnings etc here, as may be inside a generate branch that will be deleted
-	AstVar* varp = varFromBasefrom(basefromp);
+	AstNode* errp;
+	AstNodeDType* bfdtypep = baseDTypeFrom(basefromp, errp/*ref*/);
 	// SUB #'s Not needed when LSB==0 and MSB>=0 (ie [0:-13] must still get added!)
-	if (!varp->basicp()->isRanged()) {
+	if (!bfdtypep->basicp()->isRanged()) {
 	    // vector without range, or 0 lsb is ok, for example a INTEGER x; y = x[21:0];
 	    return underp;
 	} else {
-	    if (varp->basicp()->rangep()
-		&& (!varp->basicp()->rangep()->msbp()->castConst()
-		    || !varp->basicp()->rangep()->lsbp()->castConst()))
-		varp->v3fatalSrc("Non-constant variable range; errored earlier");  // in constifyParam(varp)
-	    if (varp->basicp()->littleEndian()) {
+	    if (bfdtypep->basicp()->rangep()
+		&& (!bfdtypep->basicp()->rangep()->msbp()->castConst()
+		    || !bfdtypep->basicp()->rangep()->lsbp()->castConst()))
+		bfdtypep->v3fatalSrc("Non-constant variable range; errored earlier");  // in constifyParam(bfdtypep)
+	    if (bfdtypep->basicp()->littleEndian()) {
 		// reg [1:3] was swapped to [3:1] (lsbEndianedp==3) and needs a SUB(3,under)
-		AstNode* newp = newSubNeg(varp->basicp()->msb(), underp);
+		AstNode* newp = newSubNeg(bfdtypep->basicp()->msb(), underp);
 		return newp;
 	    } else {
 		// reg [3:1] needs a SUB(under,1)
-		AstNode* newp = newSubNeg(underp, varp->basicp()->lsb());
+		AstNode* newp = newSubNeg(underp, bfdtypep->basicp()->lsb());
 		return newp;
 	    }
 	}
@@ -203,11 +221,12 @@ private:
 	int dimension      = AstArraySel::dimension(nodep->fromp());  // Not attrp as need hierarchy
 	AstNodeDType* ddtypep = dtypeForExtractp(nodep, basefromp, dimension, false);
 	AstNode* fromp = nodep->lhsp()->unlinkFrBack();
-	AstNode* bitp = nodep->rhsp()->unlinkFrBack();
+	AstNode* rhsp = nodep->rhsp()->unlinkFrBack();  // bit we're extracting
+	if (debug()>=9) ddtypep->dumpTree(cout,"-ddtypep: ");
 	if (debug()>=9) nodep->dumpTree(cout,"-vsbmd: ");
 	if (AstArrayDType* adtypep = ddtypep->castArrayDType()) {
 	    // SELBIT(array, index) -> ARRAYSEL(array, index)
-	    AstNode* subp = bitp;
+	    AstNode* subp = rhsp;
 	    if (adtypep->lsb()!=0 || adtypep->msb()<0) {
 		subp = newSubNeg (subp, adtypep->lsb());
 	    }
@@ -221,7 +240,18 @@ private:
 	    // SELBIT(range, index) -> SEL(array, index, 1)
 	    AstSel* newp = new AstSel (nodep->fileline(),
 				       fromp,
-				       newSubLsbOf(bitp, basefromp),
+				       newSubLsbOf(rhsp, basefromp),
+				       // Unsized so width from user
+				       new AstConst (nodep->fileline(),AstConst::Unsized32(),1));
+	    UINFO(6,"   new "<<newp<<endl); if (debug()>=9) newp->dumpTree(cout,"-vsbnw: ");
+	    nodep->replaceWith(newp); pushDeletep(nodep); nodep=NULL;
+	}
+	else if (AstNodeClassDType* adtypep = ddtypep->castNodeClassDType()) {
+	    if (adtypep) {} // unused
+	    // SELBIT(range, index) -> SEL(array, index, 1)
+	    AstSel* newp = new AstSel (nodep->fileline(),
+				       fromp,
+				       newSubLsbOf(rhsp, basefromp),
 				       // Unsized so width from user
 				       new AstConst (nodep->fileline(),AstConst::Unsized32(),1));
 	    UINFO(6,"   new "<<newp<<endl); if (debug()>=9) newp->dumpTree(cout,"-vsbnw: ");
@@ -232,6 +262,7 @@ private:
 	    // How to recover?  We'll strip a dimension.
 	    nodep->replaceWith(fromp); pushDeletep(nodep); nodep=NULL;
 	}
+	if (!rhsp->backp()) pushDeletep(rhsp); rhsp=NULL;
     }
 
     virtual void visit(AstSelExtract* nodep, AstNUser*) {
@@ -249,7 +280,8 @@ private:
 	AstNode* fromp = nodep->lhsp()->unlinkFrBack();
 	AstNode* msbp = nodep->rhsp()->unlinkFrBack();
 	AstNode* lsbp = nodep->thsp()->unlinkFrBack();
-	AstVar* varp = varFromBasefrom(basefromp);
+	AstNode* errp;
+	AstNodeDType* bfdtypep = baseDTypeFrom(basefromp, errp/*ref*/);
 	vlsint32_t msb = msbp->castConst()->toSInt();
 	vlsint32_t lsb = lsbp->castConst()->toSInt();
 	AstNodeDType* ddtypep = dtypeForExtractp(nodep, basefromp, dimension, msb!=lsb);
@@ -261,11 +293,11 @@ private:
 		newp->length((msb - lsb) + 1);
 		nodep->replaceWith(newp); pushDeletep(nodep); nodep=NULL;
 	    } else {
-		nodep->v3error("Illegal bit select; can't bit extract from arrayed dimension: "<<varp->prettyName());
+		nodep->v3error("Illegal bit select; can't bit extract from arrayed dimension: "<<errp->prettyName());
 	    }
 	} else if (AstBasicDType* adtypep = ddtypep->castBasicDType()) {
 	    if (adtypep) {} // Unused
-	    if (varp->basicp()->littleEndian()) {
+	    if (bfdtypep->basicp()->littleEndian()) {
 		// Below code assumes big bit endian; just works out if we swap
 		int x = msb; msb = lsb; lsb = x;
 	    }
@@ -282,13 +314,32 @@ private:
 	    UINFO(6,"   new "<<newp<<endl);
 	    //if (debug()>=9) newp->dumpTree(cout,"--SLEXnew: ");
 	    nodep->replaceWith(newp); pushDeletep(nodep); nodep=NULL;
-	    pushDeletep(msbp); msbp=NULL;
+	} else if (AstNodeClassDType* adtypep = ddtypep->castNodeClassDType()) {
+	    if (adtypep) {} // Unused
+	    // Classes aren't little endian
+	    if (lsb > msb) {
+		nodep->v3error("["<<msb<<":"<<lsb<<"] Range extract has backward bit ordering, perhaps you wanted ["<<lsb<<":"<<msb<<"]");
+		int x = msb; msb = lsb; lsb = x;
+	    }
+	    AstNode* widthp = new AstConst (msbp->fileline(), AstConst::Unsized32(), // Unsized so width from user
+					    msb +1-lsb);
+	    AstSel* newp = new AstSel (nodep->fileline(),
+				       fromp,
+				       newSubLsbOf(lsbp, basefromp),
+				       widthp);
+	    UINFO(6,"   new "<<newp<<endl);
+	    //if (debug()>=9) newp->dumpTree(cout,"--SLEXnew: ");
+	    nodep->replaceWith(newp); pushDeletep(nodep); nodep=NULL;
 	}
 	else {  // NULL=bad extract, or unknown node type
 	    nodep->v3error("Illegal range select; variable already selected, or bad dimension");
 	    // How to recover?  We'll strip a dimension.
 	    nodep->replaceWith(fromp); pushDeletep(nodep); nodep=NULL;
 	}
+	// delete whataver we didn't use in reconstruction
+	if (!fromp->backp()) pushDeletep(fromp); fromp=NULL;
+	if (!msbp->backp()) pushDeletep(msbp); msbp=NULL;
+	if (!lsbp->backp()) pushDeletep(lsbp); lsbp=NULL;
     }
 
     void replaceSelPlusMinus(AstNodePreSel* nodep) {
@@ -296,14 +347,13 @@ private:
 	// Below 2 lines may change nodep->widthp()
 	V3Const::constifyParamsEdit(nodep->thsp()); // May relink pointed to node
 	checkConstantOrReplace(nodep->thsp(), "Width of :+ or :- bit extract isn't a constant");
-	// Now replace it with a AstSel
+	// Now replace it with an AstSel
 	AstNode* basefromp = AstArraySel::baseFromp(nodep->attrp());
 	int dimension      = AstArraySel::dimension(nodep->fromp());  // Not attrp as need hierarchy
 	AstNode* fromp = nodep->lhsp()->unlinkFrBack();
 	AstNode* rhsp  = nodep->rhsp()->unlinkFrBack();
 	AstNode* widthp = nodep->thsp()->unlinkFrBack();
 	int width = widthp->castConst()->toSInt();
-	//AstVar* varp = varFromBasefrom(basefromp);
 	if (width > (1<<28)) nodep->v3error("Width of :+ or :- is huge; vector of over 1billion bits: "<<widthp->prettyName());
 	if (width<0) nodep->v3error("Width of :+ or :- is < 0: "<<widthp->prettyName());
 	AstNodeDType* ddtypep = dtypeForExtractp(nodep, basefromp, dimension, width!=1);
@@ -342,6 +392,10 @@ private:
 	    }
 	    UINFO(6,"   new "<<newp<<endl);
 	    nodep->replaceWith(newp); pushDeletep(nodep); nodep=NULL;
+	    // delete whataver we didn't use in reconstruction
+	    if (!fromp->backp()) pushDeletep(fromp); fromp=NULL;
+	    if (!rhsp->backp()) pushDeletep(rhsp); rhsp=NULL;
+	    if (!widthp->backp()) pushDeletep(widthp); widthp=NULL;
 	}
 	else {  // NULL=bad extract, or unknown node type
 	    nodep->v3error("Illegal +: or -: select; variable already selected, or bad dimension");
@@ -376,6 +430,7 @@ public:
 // Width class functions
 
 AstNode* V3Width::widthSelNoIterEdit(AstNode* nodep) {
+    UINFO(4,__FUNCTION__<<": "<<nodep<<endl);
     WidthSelVisitor visitor;
     nodep = visitor.mainAcceptEdit(nodep);
     return nodep;
