@@ -60,6 +60,7 @@ my $opt_vlt;
 my $opt_vcs;
 my $opt_verbose;
 my $Opt_Verilated_Debug;
+our $Opt_Verilation = 1;
 our @Opt_Driver_Verilator_Flags;
 
 Getopt::Long::config ("pass_through");
@@ -86,6 +87,7 @@ if (! GetOptions (
 		  "vlt!"	=> \$opt_vlt,
 		  "vcs!"	=> \$opt_vcs,
 		  "verbose!"	=> \$opt_verbose,
+		  "verilation!"		=> \$Opt_Verilation,  # Undocumented debugging
 		  "verilated_debug!"	=> \$Opt_Verilated_Debug,
 	  	  #W		   see parameter()
 		  "<>"		=> \&parameter,
@@ -169,10 +171,10 @@ sub one_test {
 		 $UnsupCnt++;
 	     } else {
 		 $test->oprint("FAILED: ","*"x60,"\n");
-		 push @fails, "\t#".$test->soprint("%Error: $test->{errors}\n");
 		 my $j = ($opt_jobs>1?" -j":"");
-		 push @fails, "\t\tmake$j && test_regress/"
-		     .$test->{pl_filename}." ".join(' ',@Orig_ARGV_Sw)."\n";
+		 push @fails, ("\t#".$test->soprint("%Error: $test->{errors}\n")
+			       ."\t\tmake$j && test_regress/"
+			       .$test->{pl_filename}." ".join(' ',@Orig_ARGV_Sw)."\n");
 		 $FailCnt++;
 		 report(\@fails, $Log_Filename);
 		 my $other = "";
@@ -202,7 +204,7 @@ sub usage {
 
 sub debug {
     $Debug = 1;
-    push @Opt_Driver_Verilator_Flags, "--debug";
+    push @Opt_Driver_Verilator_Flags, "--debug --no-skip-identical";
 }
 
 our $_Parameter_Next_Level;
@@ -254,7 +256,7 @@ sub report {
     $fh->print("="x70,"\n");
     $fh->printf("TESTS Passed $OkCnt  Unsup $UnsupCnt  Skipped $SkipCnt  Failed $FailCnt  Time %d:%02d\n",
 	       int($delta/60),$delta%60);
-    foreach my $f (@$fails) {
+    foreach my $f (sort @$fails) {
 	chomp $f;
 	$fh->print("$f\n");
     }
@@ -352,7 +354,8 @@ sub new {
 	vcs_run_flags => [split(/\s+/,"+vcs+lic_wait")],
 	# NC
 	nc => 0,
-	nc_flags => [split(/\s+/,"+licqueue +nowarn+LIBNOU +define+NC=1 -q +assert +sv -c ")],
+	nc_flags => [split(/\s+/,("+licqueue +nowarn+LIBNOU +define+NC=1 -q +assert +sv -c "
+				  .($opt_trace ? " +access+r":"")))],
 	nc_flags2 => [],  # Overridden in some sim files
 	nc_run_flags => [split(/\s+/,"+licqueue -q +assert +sv -R")],
 	# Verilator
@@ -473,6 +476,7 @@ sub compile_vlt_flags {
     $self->{sc} = 1 if ($checkflags =~ /-sc\b/);
     $self->{sp} = 1 if ($checkflags =~ /-sp\b/);
     $self->{trace} = 1 if ($opt_trace || $checkflags =~ /-trace\b/);
+    $self->{savable} = 1 if ($checkflags =~ /-savable\b/);
     $self->{coverage} = 1 if ($checkflags =~ /-coverage\b/);
 
     my @verilator_flags = @{$param{verilator_flags}};
@@ -627,7 +631,7 @@ sub compile {
 	$self->_run(logfile=>"$self->{obj_dir}/vlt_compile.log",
 		    fails=>$param{fails},
 		    expect=>$param{expect},
-		    cmd=>\@cmdargs);
+		    cmd=>\@cmdargs) if $::Opt_Verilation;
 	return 1 if $self->errors || $self->skips || $self->unsupporteds;
 
 	if (!$param{fails} && $param{verilator_make_gcc}) {
@@ -960,15 +964,39 @@ sub _make_main {
     print $fh "#include \"systemc.h\"\n" if $self->sc;
     print $fh "#include \"systemperl.h\"\n" if $self->sp;
     print $fh "#include \"verilated_vcd_c.h\"\n" if $self->{trace} && !$self->sp;
+    print $fh "#include \"verilated_save.h\"\n" if $self->{savable};
     print $fh "#include \"SpTraceVcd.h\"\n" if $self->{trace} && $self->sp;
 
     print $fh "$VM_PREFIX * topp;\n";
     if (!$self->sc_or_sp) {
-	print $fh "unsigned int main_time = false;\n";
+	print $fh "vluint64_t main_time = false;\n";
 	print $fh "double sc_time_stamp () {\n";
 	print $fh "    return main_time;\n";
 	print $fh "}\n";
     }
+
+    if ($self->{savable}) {
+	$fh->print("\n");
+	$fh->print("void save_model(const char* filenamep) {\n");
+	$fh->print("    VL_PRINTF(\"Saving model to '%s'\\n\", filenamep);\n");
+	$fh->print("    VerilatedSave os;\n");
+	$fh->print("    os.open(filenamep);\n");
+	$fh->print("    os << main_time;\n");
+	$fh->print("    os << *topp;\n");
+	$fh->print("    os.close();\n");
+	$fh->print("}\n");
+	$fh->print("\n");
+	$fh->print("void restore_model(const char* filenamep) {\n");
+	$fh->print("    VL_PRINTF(\"Restoring model from '%s'\\n\", filenamep);\n");
+	$fh->print("    VerilatedRestore os;\n");
+	$fh->print("    os.open(filenamep);\n");
+	$fh->print("    os >> main_time;\n");
+	$fh->print("    os >> *topp;\n");
+	$fh->print("    os.close();\n");
+	$fh->print("}\n");
+    }
+
+    #### Main
     if ($self->sc_or_sp) {
 	print $fh "extern int sc_main(int argc, char **argv);\n";
 	print $fh "int sc_main(int argc, char **argv) {\n";
@@ -1008,12 +1036,30 @@ sub _make_main {
 	}
 	$fh->print("    topp->trace (tfp, 99);\n");
 	$fh->print("    tfp->open (\"$self->{obj_dir}/simx.vcd\");\n");
+	if ($self->{trace} && !$self->sc_or_sp) {
+	    $fh->print("	if (tfp) tfp->dump (main_time);\n");
+	}
 	$fh->print("#endif\n");
     }
 
-    print $fh "    ${set}fastclk = false;\n" if $self->{inputs}{fastclk};
-    print $fh "    ${set}clk = false;\n" if $self->{inputs}{clk};
+    if ($self->{savable}) {
+	$fh->print("    const char* save_time_strp  = Verilated::commandArgsPlusMatch(\"save_time=\");\n");
+	$fh->print("    const char* save_restore_strp = Verilated::commandArgsPlusMatch(\"save_restore=\");\n");
+	$fh->print("    unsigned int save_time = !save_time_strp[0] ? 0 : atoi(save_time_strp+strlen(\"+save_time=\"));\n");
+	$fh->print("    unsigned int save_restore = !save_restore_strp[0] ? 0 : 1;\n");
+    }
+
+    if ($self->{savable}) {
+	$fh->print("    if (save_restore) {\n");
+	$fh->print("        restore_model(\"$self->{obj_dir}/saved.vltsv\");\n");
+	$fh->print("    } else {\n");
+    } else {
+	$fh->print("    {\n");
+    }
+    print $fh "       ${set}fastclk = false;\n" if $self->{inputs}{fastclk};
+    print $fh "       ${set}clk = false;\n" if $self->{inputs}{clk};
     _print_advance_time($self, $fh, 10);
+    print $fh "    }\n";
 
     print $fh "    while (sc_time_stamp() < sim_time && !Verilated::gotFinish()) {\n";
     for (my $i=0; $i<5; $i++) {
@@ -1026,6 +1072,13 @@ sub _make_main {
 	    print $fh "	${set}clk=!${set}clk;\n";
 	    $action = 1;
 	}
+	if ($self->{savable}) {
+	    $fh->print("	if (sc_time_stamp() == save_time && save_time) {\n");
+	    $fh->print("	    save_model(\"$self->{obj_dir}/saved.vltsv\");\n");
+	    $fh->print("	    printf(\"Exiting after save_model\\n\");\n");
+	    $fh->print("	    exit(0);\n");
+	    $fh->print("	}\n");
+	}
 	_print_advance_time($self, $fh, 1, $action);
     }
     print $fh "    }\n";
@@ -1072,7 +1125,7 @@ sub _print_advance_time {
     } else {
 	if ($action) {
 	    print $fh "	${set}eval();\n";
-	    if ($self->{trace} && !$self->{sp}) {
+	    if ($self->{trace} && !$self->sc_or_sp) {
 		$fh->print("#if VM_TRACE\n");
 		$fh->print("	if (tfp) tfp->dump (main_time);\n");
 		$fh->print("#endif //VM_TRACE\n");
@@ -1360,7 +1413,9 @@ sub file_grep_not {
     my $self = (ref $_[0]? shift : $Self);
     my $filename = shift;
     my $regexp = shift;
+    my $expvalue = shift;
     return if $self->errors || $self->skips || $self->unsupporteds;
+    !defined $expvalue or $self->error("file_grep_not: Unexpected 3rd argument: $expvalue");
 
     my $contents = $self->file_contents($filename);
     return if ($contents eq "_Already_Errored_");
@@ -1373,12 +1428,15 @@ sub file_grep {
     my $self = (ref $_[0]? shift : $Self);
     my $filename = shift;
     my $regexp = shift;
+    my $expvalue = shift;
     return if $self->errors || $self->skips || $self->unsupporteds;
 
     my $contents = $self->file_contents($filename);
     return if ($contents eq "_Already_Errored_");
     if ($contents !~ /$regexp/) {
 	$self->error("File_grep: $filename: Regexp not found: $regexp\n");
+    } elsif ($expvalue && $expvalue ne $1) {
+	$self->error("File_grep: $filename: Got='$1' Expected='$expvalue' in regexp: $regexp\n");
     }
 }
 
@@ -1467,27 +1525,226 @@ The driver reports the number of tests which pass, fail, skipped (some
 resource required by the test is not available, such as SystemC), or are
 unsupported (buggy or require a feature change before will pass.)
 
-=head1 ARGUMENTS
+There are hundreds of tests, and for faster completion you may want to run
+the regression tests with CCACHE enabled and in parallel on a machine with
+many cores.  See the -j option.
+
+=head1 TEST CONFIGURATION
+
+The Perl script (e.g. C<test_regres/t/t_EXAMPLE.pl>) controls how the test
+will run by driver.pl. In general it includes a call to the C<compile>
+subroutine to compile the test with Verilator (or an alternative
+simulator), followed by a call to the C<execute> subroutine to run the
+test. Compile-only tests omit the call to C<execute>.
+
+If those complete, the script calls C<ok(1)> to increment the count of
+successful tests and then returns 1 as its result.
+
+Both C<compile> and C<execute> take an optional argument hash table to
+control their behavior. For example:
+
+  compile (
+     verilator_flags2 => ["--lint-only"],
+     fails => 1,
+  );
+
+indicates that when compiling this test, the C<--lint-only> flag should be
+passed and that the test is expected to fail.
+
+The full list of arguments can be found by looking at the C<driver.pl>
+source code. Some commonly used arguments are:
+
+=over 4
+
+=item all_run_flags
+
+A list of flags to be passed when running the simulator (Verilated model or
+one of the other simulators).
+
+=item check_finished
+
+Set to 1 to indicate successful completion of the test is indicated by the
+string C<*-* All Finished *-*> being printed on standard output. This is
+the normal way for successful tests to finish.
+
+=item expect
+
+A quoted list of strings or regular expression to be matched in the
+output. See </HINTS ON WRITING TESTS> for more detail on how this argument
+should be used.
+
+=item fails
+
+Set to 1 to indicate this step (C<compile> or C<execute>) is expected to
+fail.  Tests that are expected to fail generally have _bad in their
+filename.
+
+=item make_main
+
+Set to 0 to disable the automatic creation of a C++ test wrapper (for
+example when a hand-written test wrapper is provided using C<--exe>).
+
+=item make_top_shell
+
+Set to 0 to disable the automatic creation of a top level shell to run the
+executable (for example when a hand-written test wrapper is provided using
+C<--exe>).
+
+=item nc_flags
+
+=item nc_flags2
+
+=item nc_run_flags
+
+The equivalent of C<v_flags>, C<v_flags2> and C<all_run_flags>, but only
+for use with the Cadence NC simulator.
+
+=item iv_flags
+
+=item iv_flags2
+
+=item iv_run_flags
+
+The equivalent of C<v_flags>, C<v_flags2> and C<all_run_flags>, but only
+for use with the Icarus Verilog simulator.
+
+=item v_flags
+
+A list of standard Verilog simulator flags to be passed to the simulator
+compiler (Verilator or one of the other simulators).  This list is create
+by the driver and rarely changed, use v_flags2 instead.
+
+=item v_flags2
+
+A list of standard Verilog simulator flags to be passed to the simulator
+compiler (Verilator or one of the other simulators). Unlike v_flags, these
+options may be overridden in some simulation files.
+
+Similar sets of flags exist for atsim, GHDL, Cadence NC and Synopsys VCS.
+
+=item vcs_flags
+
+=item vcs_flags2
+
+=item vcs_run_flags
+
+The equivalent of C<v_flags>, C<v_flags2> and C<all_run_flags>, but only
+for use with the Synopsys VCS simulator.
+
+=item verilator_flags
+
+=item verilator_flags2
+
+The equivalent of C<v_flags> and C<v_flags2>, but only for use with
+Verilator.  If a flag is a standard flag (+incdir for example) v_flags2
+should be used instead.
+
+=back
+
+=head2 HINTS ON WRITING TESTS
+
+There is generally no need for the test to create its own main program or
+top level shell as the driver creates one automatically, however some tests
+require their own C++ or SystemC test harness. This is commonly given the
+same name as the test, but with .cpp as suffix
+(C<test_regress/t/t_EXAMPLE.cpp>). This can be specified as follows:
+
+  compile (
+      make_top_shell   => 0,
+      make_main        => 0,
+      verilator_flags2 => ["--exe $Self->{t_dir}/$Self->{name}.cpp"], );
+
+Tests should be self-checking, rather than producing lots of output. If a
+test succeeds it should print C<*-* All Finished *-*> to standard output
+and terminate (in Verilog C<$finish>), if not it should just stop (in
+Verilog C<$stop>) as that signals an error.
+
+If termination should be triggered from the C++ wrapper, the following code
+can be used:
+
+  vl_fatal (__FILE__, __LINE__, "dut", "<error message goes here>");
+  exit (1);
+
+This can be particularly useful if checking that the Verilator model has
+not unexpectedly terminated.
+
+  if (Verilated::gotFinish ()) {
+      vl_fatal (__FILE__, __LINE__, "dut", "<error message goes here>");
+      exit (1);
+  }
+
+Where it might be useful for a test to produce output, it should qualify
+this with C<TEST_VERBOSE>. For example in Verilog:
+
+  `ifdef TEST_VERBOSE
+        $write ("Conditional generate if MASK [%1d] = %d\n", g, MASK[g]);
+  `endif
+
+Or in a hand-written C++ wrapper:
+
+  #ifdef TEST_VERBOSE
+      cout << "Read  a     = " << a << endl;
+  #endif
+
+The C<expect> argument should not generally be used to decide if a test has
+succeeded. However, in the case of tests that are designed to fail at
+compile time, it is the only option. For example:
+
+  compile (
+           v_flags2 => ["--lint-only"],
+           fails=>1,
+           expect=>
+  q{%Error: t/t_gen_cond_bitrange_bad.v:\d+: Selection index out of range: 2:2 outside 1:0
+  %Error: t/t_gen_cond_bitrange_bad.v:\d+: Selection index out of range: 2:2 outside 1:0
+  %Error: t/t_gen_cond_bitrange_bad.v:\d+: Selection index out of range: 2:2 outside 1:0
+  %Error: t/t_gen_cond_bitrange_bad.v:\d+: Selection index out of range: 2:2 outside 1:0
+  %Error: Exiting due to .*},
+           );
+
+The strings to match should be made general - for example the line numbers
+in the Verilog should not be critical and the total number of errors should
+not matter. This makes it easier to extend or modify the test in future.
+
+=head1 DRIVER ARGUMENTS
 
 =over 4
 
 =item --atsim
 
-Run using ATSIM.
+Run using ATSIM simulator.
 
 =item --benchmark [<cycles>]
 
 Show execution times of each step.  If an optional number is given,
 specifies the number of simulation cycles (for tests that support it).
 
+=item --debug
+
+Same as C<verilator --debug>: Use the debug version of Verilator which
+enables additional assertions, debugging messages, and structure dump
+files.
+
+=item --debugi(-<srcfile>) <level>
+
+Same as C<verilator --debugi level>: Set Verilator internal debugging level
+globally to the specified debug level (1-10) or set the specified source
+file to the specified level. Higher levels produce more detailed messages
+(plain C<--debug> is equivalent to C<--debugi 4>).
+
+=item --dump-tree
+
+Same as C<verilator --dump-tree>: Enable Verilator writing .tree debug
+files with dumping level 3, which dumps the standard critical stages.  For
+details on the format see the Verilator Internals manual.
+
 =item --gdb
 
-Run Verilator under the debugger.
+Same as C<verilator --gdb>: Run Verilator under the debugger.
 
 =item --gdbbt
 
-Run Verilator under the debugger, only to print backtrace information.
-Requires --debug.
+Same as C<verilator --gdbbt>: Run Verilator under the debugger, only to
+print backtrace information.  Requires --debug.
 
 =item --gdbsim
 
@@ -1495,7 +1752,7 @@ Run Verilator generated executable under the debugger.
 
 =item --ghdl
 
-Run using GHDL.
+Run using GHDL simulator.
 
 =item --golden
 
@@ -1505,20 +1762,24 @@ Update golden files, equivalent to setting HARNESS_UPDATE_GOLDEN=1.
 
 Displays this message and program version and exits.
 
+=item --iverilog
+
+Run using Icarus Verilog simulator.
+
 =item --j #
 
 Run number of parallel tests, or 0 to determine the count based on the
 number of cores installed.  Requires Perl's Parallel::Forker package.
 
+=item --nc
+
+Run using Cadence NC-Verilog simulator.
+
 =item --optimize
 
 Randomly turn on/off different optimizations.  With specific flags,
 use those optimization settings
 
-=item --nc
-
-Run using NC-Verilog.
-
 =item --site
 
 Run site specific tests also.
@@ -1527,17 +1788,22 @@ Run site specific tests also.
 
 Stop on the first error.
 
+=item --trace
+
+Set the simulator specific flags to request waveform tracing.
+
 =item --vcs
 
-Run using VCS.
+Run using Synopsys VCS simulator.
 
 =item --verbose
 
-Enable test verbose messages.
+Compile and run the test in verbose mode. This means C<TEST_VERBOSE> will
+be defined for the test (Verilog and any C++/SystemC wrapper).
 
 =item --vlt
 
-Run using Verilator.
+Run using Verilator.  Defaults set unless another simulator is requested.
 
 =back
 
